﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Security;
using System.Threading;
using SlimDX;
using SlimDX.Direct3D9;
using SlimDX.Windows;
using SlimDX.DirectInput;
using FDK;
using DirectShowLib;

namespace StrokeStyleT
{
	class CApp : CApplicationForm
	{
		/// <summary>
		/// <para>Global.App.Window.Handle　を進行スレッドにて使用すると例外が出る。（コントロール作成スレッドではないため。）</para>
		/// <para>そのため、ウィンドウハンドルをここに控えておく。</para>
		/// </summary>
		public IntPtr hWnd
		{
			get;
			protected set;
		}
		
		// 以下の２つはユーザ別設定ウィンドウにて立てられるフラグ。
		public volatile bool bD3Dデバイスを変更する = false;
		public volatile bool bサウンドデバイスを変更する = false;

		public CApp()
		{
			Global.t初期化( this );		// Program.Main() から参照されるので、先にこれだけ初期化。
		}


		// オーバーライドメソッド 

		/// <summary>
		/// <para>メインウィンドウの生成と各種初期化を行う。</para>
		/// <para>Direct3D の生成の後に呼び出される。</para>
		/// <para>エラー等でアプリを終了したい場合は例外を発生させ、正常に（無言で）終了したい場合は this.Window を null にして return すること。</para>
		/// </summary>
		protected override void On初期化()
		{
			#region [ プライマリアダプタのHALとフォーマットのチェックを行う。]
			//-----------------
			if( !this.Direct3D.CheckDeviceType( 0, SlimDX.Direct3D9.DeviceType.Hardware, Format.X8R8G8B8, Format.X8R8G8B8, true ) ||	// ウィンドウモード
				!this.Direct3D.CheckDeviceType( 0, SlimDX.Direct3D9.DeviceType.Hardware, Format.X8R8G8B8, Format.X8R8G8B8, false ) )	// 全画面モード
			{
				string msg = "プライマリディスプレイアダプタが、本ソフトの動作に必要な機能を満たしていません。";
				MessageBox.Show( msg, "StrokeStyle<T> エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
				throw new Exception( msg );
			}
			//-----------------
			#endregion
			#region [ Global.* を初期化する。--> コンストラクタへ移動 ]
			//-----------------
			//Global.t初期化( this );	
			//-----------------
			#endregion

			#region [ ウィンドウを生成する。]
			//-----------------
			this.Window = new Cメインウィンドウ( this ) {
				Text = ( Global.PlayerMode.bプレイヤーモードである ) ?
					Properties.Resources.TITLE_PLAYERMODE :
					string.Format( "{0} {1}.{2} (rev.{3} build.{4})", Properties.Resources.TITLE_NORMALMODE, Global.nメジャー番号, Global.nマイナー番号, Global.nリビジョン番号, Global.nビルド番号 ),
				ClientSize = Global.Theme.WindowClientSize,
				MinimizeBox = false,
				MaximizeBox = false,
			};
			this.Window.Activated += ( o, args ) => { lock( this.obj排他用 ) { Global.bウィンドウがアクティブである = true; } };
			this.Window.Deactivate += ( o, args ) => { lock( this.obj排他用 ) { Global.bウィンドウがアクティブである = false; } };
			this.Window.ResizeEnd += new EventHandler( this.OnResizeEnd );

			this.hWnd = this.Window.Handle;
			//-----------------
			#endregion
		}

		/// <summary>
		/// <para>Direct3Dデバイス（this.Device）に対するデフォルト設定を行う。</para>
		/// <para>Direct3Dデバイスのリセット・変更・再作成時に呼び出される。</para>
		/// </summary>
		protected override void OnD3Dデバイスステータスの初期化()
		{
			Size sz論理画面 = Global.Theme.LogicalDisplaySize;
			Size sz物理画面 = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize : Global.Theme.FullscreenSize;

			float f視野角 = 45.0f;		// [度]

			// ビュー行列の設定
			D3D9Device.SetTransform( TransformState.View,
				Matrix.LookAtLH(
					new Vector3( 0f, 0f, (float) ( ( -sz物理画面.Height / 2.0 ) / Math.Tan( C共通.DegreeToRadian( f視野角 / 2.0 ) ) ) ),	// カメラの位置
					new Vector3( 0f, 0f, 0f ),																	// 見ているターゲット
					new Vector3( 0f, 1f, 0f ) ) );																// 上方向

			// 射影行列の設定
			D3D9Device.SetTransform( TransformState.Projection,
				Matrix.PerspectiveFovLH(
					C共通.DegreeToRadian( f視野角 ),															// y 方向の視野角度（ラジアン）
					(float) sz物理画面.Width / sz物理画面.Height,												// アスペクト比。View の幅÷高さ。
					-100.0f,																					// z（近）
					100.0f ) );																					// z（遠）

			D3D9Device.SetRenderState( RenderState.Lighting, false );
			D3D9Device.SetRenderState( RenderState.ZEnable, false );
			D3D9Device.SetRenderState( RenderState.AntialiasedLineEnable, false );
			D3D9Device.SetRenderState( RenderState.AlphaTestEnable, true );
			D3D9Device.SetRenderState( RenderState.AlphaRef, 10 );
			D3D9Device.SetRenderState( RenderState.MultisampleAntialias, false );
			D3D9Device.SetRenderState<Compare>( RenderState.AlphaFunc, Compare.Greater );
			D3D9Device.SetRenderState( RenderState.AlphaBlendEnable, true );
			D3D9Device.SetRenderState<Blend>( RenderState.SourceBlend, Blend.SourceAlpha );
			D3D9Device.SetRenderState<Blend>( RenderState.DestinationBlend, Blend.InverseSourceAlpha );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaOperation, TextureOperation.Modulate );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaArg1, 2 );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaArg2, 1 );
		}
		


		/// <summary>
		/// <para>WindowsGUIの描画処理を行う。</para>
		/// <para>ロックを得たGUIスレッドにより実行される。</para>
		/// <para>BeginScene() と EndScene() の外（On描画の直後）に呼び出される。</para>
		/// <para>そのため、Direct3Dデバイスへの描画は行わないこと。</para>
		/// </summary>
		protected override void OnGUI()
		{
			#region [ プレイヤーモードの場合、コマンドライン配列を定期的にチェックし、必要有ればアクションを実行する。]
			//-----------------
			if( (Global.PlayerMode.bプレイヤーモードである) && (Global.PlayerMode.arrayコマンドライン != null ) )		// 非 null なら実行。
			{
				var e結果 = Global.PlayerMode.tアクション解析・指示( Global.PlayerMode.arrayコマンドライン );

				Global.PlayerMode.arrayコマンドライン = null;		// 解析が終わったら、アクション実行前に null に戻すこと。（でないとまたこのif文にひっかかる）

				if( e結果 != E状態処理結果.OK )		// 指示があった → アクションを実行へ移す（遷移する）
				{
					this.e状態処理結果 = e結果;
					this.t全完了();
					return;
				}
			}
			//-----------------
			#endregion

			switch( this.e現在の状態[ THREAD_GUI ] )
			{
				#region [ 待機 ]
				//-----------------
				case Eアプリ状態.待機:
					break;
				//-----------------
				#endregion
				#region [ アプリ起動 ]
				//-----------------
				case Eアプリ状態.アプリ起動:
					{
						Global.Input.Input管理 = new CInput管理( this.Window.Handle );
						this.tトップレベルActivityを登録();
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ログイン名取得 ]
				//-----------------
				case Eアプリ状態.ログイン名取得:
					{
						this.str次のログインユーザ名 = null;

						#region [ 現状ユーザ数が 0 人、または SSTFEditor用ユーザが存在していないなら、SSTFEditor用ユーザを作成する。]
						//-----------------
						bool bSSTFEditor用ユーザを作成する = false;

						#region [ SSTFEditor用ユーザの作成は必要か？ ]
						//-----------------
						if( Global.Users.User.Count == 0 )
						{
							bSSTFEditor用ユーザを作成する = true;
						}
						else
						{
							bSSTFEditor用ユーザを作成する = true;
							foreach( var name in Global.Users.User )
							{
								if( name.Equals( Properties.Resources.DEFAULT_VIEWER_USER_NAME, StringComparison.OrdinalIgnoreCase ) )
								{
									bSSTFEditor用ユーザを作成する = false;		// ユーザがあった（ので作らない）
									break;
								}
							}
						}
						//-----------------
						#endregion

						if( bSSTFEditor用ユーザを作成する )
						{
							#region [ 必要だったので作成する。]
							//-----------------
							try
							{
								Global.Users.tユーザを新規追加しXMLファイルを保存する( Properties.Resources.DEFAULT_VIEWER_USER_NAME, Global.Folder.strUsersXMLの絶対パス );
							}
							catch( Exception e )
							{
								MessageBox.Show( "ユーザの登録に失敗しました。終了します。\n--------\n" + e.ToString(), "StrokeStyle<T>エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
								throw;
							}
							//-----------------
							#endregion
						}
						//-----------------
						#endregion

						#region [ 最初のユーザを取得し、this.str次のユーザ名に格納する。]
						//-----------------
						if( Global.PlayerMode.bプレイヤーモードである )
						{
							this.str次のログインユーザ名 = Properties.Resources.DEFAULT_VIEWER_USER_NAME;
							this.e状態処理結果 = (int) E状態処理結果.OK;
						}
						else
						{
							if( Global.Users.User.Count == 1 )	// SSTFEditor しかいない場合
							{
								#region [ ユーザ登録：最初のユーザ名を起動者に入力させる。]
								//-----------------
								this.str次のログインユーザ名 = Global.Users.tユーザ登録ウィンドウを表示し入力されたユーザ名を返す( this.Window );
								this.e状態処理結果 = (int) E状態処理結果.OK;

								if( string.IsNullOrEmpty( this.str次のログインユーザ名 ) )	// キャンセルされた
									this.e状態処理結果 = E状態処理結果.CANCEL;
								//-----------------
								#endregion
							}
							else
							{
								#region [ ユーザ選択：既存ユーザを選択させる。]
								//-----------------
								this.str次のログインユーザ名 = Global.Users.tユーザ管理ウィンドウを表示し選択されたユーザ名を返す( this.Window );
								this.e状態処理結果 = (int) E状態処理結果.OK;

								if( string.IsNullOrEmpty( this.str次のログインユーザ名 ) )	// キャンセルされた
									this.e状態処理結果 = E状態処理結果.CANCEL;
								//-----------------
								#endregion
							}
						}
						//-----------------
						#endregion
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ログイン ]
				//-----------------
				case Eアプリ状態.ログイン:
					{
						Global.Users.str現在のユーザ名 = this.str次のログインユーザ名;
						Global.Folder.tユーザ個別フォルダを変更する( Global.Users.str現在のユーザ名 );

						#region[ ユーザの Config.xml を読み込む。]
						//-----------------
						Global.Users.Config = Config.t読み込む( Global.Folder.strConfigXMLの絶対パス );

						// プレイヤーモードなら AutoPlay をすべて ON にする。
						// （プレイヤーモードではConfigを保存しないので、好きに変更しても大丈夫。）

						if( Global.PlayerMode.bプレイヤーモードである )
							Global.Users.Config.tすべてのAutoPlayを設定する( true );
						//-----------------
						#endregion

						#region [ Traceリスナを追加する。（ログ出力する場合のみ）]
						//-----------------
						Trace.AutoFlush = true;
						Trace.IndentLevel = 0;

						if( Global.Users.Config.WriteLog )	// ログ出力する場合のみ
						{
							string path = Path.Combine( Global.Folder.strユーザ個別フォルダ, Properties.Resources.LOG_FILE_NAME );

							try
							{
								Trace.Listeners.Add(
									new CTraceLogListener(
										new StreamWriter( path, false, Encoding.GetEncoding( "shift-jis" ) ) ) );
							}
							catch( Exception e )
							{
								MessageBox.Show( string.Format( "{0} へのリスナ追加に失敗しました。\nログファイルは出力または上書きされません。\n---------\n{1}", path, e.ToString() ), "エラー", MessageBoxButtons.OK );
								//throw;	終了しない。
							}
						}
						//-----------------
						#endregion
						#region [ ログにタイトルを出力する。]
						//-----------------
						Trace.WriteLine( "" );
						Trace.Write( ( Global.PlayerMode.bプレイヤーモードである ) ? Properties.Resources.TITLE_PLAYERMODE : Properties.Resources.TITLE_NORMALMODE );
						Trace.WriteLine( string.Format( " {0}.{1} (rev.{2} build.{3})", Global.nメジャー番号, Global.nマイナー番号, Global.nリビジョン番号, Global.nビルド番号 ) );
						Trace.WriteLine( "Copyright 2009 2011 (C) ProjectSST All Rights Reserved." );
						Trace.WriteLine( "" );
						Trace.WriteLine( string.Format( "ユーザ「{0}」でログインします。", Global.Users.str現在のユーザ名 ) );
						Trace.WriteLine( "" );
						//-----------------
						#endregion

						Global.t現在のユーザConfigに従ってサウンドデバイスとすべての既存サウンドを再構築する();		// DirectSound が hWnd を必要とするので、ウィンドウ生成後に呼び出すこと。
						Global.Stage.m演奏ステージ.t演奏レーンチップ対応表をConfigに合わせて再構築する();

						#region[ Theme の構築（サウンドデバイス生成後に呼び出すこと。） ]
						//-----------------
						// 古いThemeがあれば解放する。（最初のログイン時が該当）
						if( Global.Theme != null )
							C共通.tDisposeする( Global.Theme );

						// Theme.xml を読み込み、初期化する。
						Global.Theme = Theme.t読み込む( Global.Theme.tファイルパス( Properties.Resources.XMLNAME_THEME ) );
						Global.Theme.t初期化( this.Direct3D );

						// プレイヤーモード時においては強制的にウィンドウモードとする。
						if( Global.PlayerMode.bプレイヤーモードである )
							Global.Users.Config.Windowed = true;

						// Config から必要なパラメータを反映する。
						if( Global.Users.Config.WindowClientSize.Width > 0 ||
							Global.Users.Config.WindowClientSize.Height > 0 )
						{
							Global.Theme.WindowClientSize = Global.Users.Config.WindowClientSize;
						}
						if( Global.Users.Config.FullscreenSize.Width > 0 ||
							Global.Users.Config.FullscreenSize.Height > 0 )
						{
							Global.Theme.FullscreenSize = Global.Users.Config.FullscreenSize;
						}
						if( Global.Users.Config.FullscreenRefreshRate > 0 )
							Global.Theme.FullscreenRefreshRate = Global.Users.Config.FullscreenRefreshRate;
						//-----------------
						#endregion

						#region [ Direct3Dデバイスのリセットまたは新規生成 ]
						//-----------------
						this.tDirect3Dデバイスを生成・変更・リセットする(
							new CD3DSettings() {
								nAdaptor = 0,
								DeviceType = SlimDX.Direct3D9.DeviceType.Hardware,
								CreateFlags = CreateFlags.HardwareVertexProcessing,
								PresentParameters = new PresentParameters() {
									BackBufferCount = 1,
									BackBufferWidth = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Width : Global.Theme.FullscreenSize.Width,
									BackBufferHeight = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Height : Global.Theme.FullscreenSize.Height,
									Windowed = Global.Users.Config.Windowed,
									BackBufferFormat = Global.Theme.BackbufferFormat,	// GUI を使う場合、GDIと互換性のあるフォーマットでなければならない。
									DeviceWindowHandle = this.Window.Handle,
									Multisample = MultisampleType.None,					// GDI を使う場合、None でなければならない。
									SwapEffect = SwapEffect.Discard,					// GUI を使う場合、Discard でなければならない。
									PresentationInterval = ( Global.Users.Config.VSyncWait ) ? PresentInterval.Default : PresentInterval.Immediate,
									PresentFlags = PresentFlags.LockableBackBuffer,		// GDI を使う場合、LockableBackBuffer でなければならない。
								}
							},
							Global.Theme.LogicalDisplaySize,
							CApplicationForm.wsウィンドウスタイル,
							CApplicationForm.ws全画面スタイル,
							true );
						//-----------------
						#endregion

						#region[ 曲リストを読み込む。]
						//-----------------
						if( Global.PlayerMode.bプレイヤーモードではない )
						{
							Global.Song.t曲リストをクリアしConfigに記されているSongListをすべて読み込んでマージする();
							Global.Stage.m選曲ステージ.t選択曲を最初の曲にリセットする();
						}
						else
							Global.Song.list曲リスト = new List<SongNode>();	// プレイヤーモードでは曲リストを構築しない。
						//-----------------
						#endregion

						#region [ 仮想ドラムサウンドを構築する。]
						//-----------------
						if( Global.Users.Config.KitListFiles.Count > 0 )
						{
							// (A) KitListFiles 要素があった場合 → 先頭にあるキットを選択。今のところ複数キット対応は未実装。
							string fileName = Global.Users.Config.KitListFiles[ 0 ];
							Global.VirtualDrums.KitList = KitList.t読み込む( fileName );
						}
						else
						{
							// (B) KitListFiles 要素がなかった場合 → 規定のドラムを使用。
							Global.VirtualDrums.KitList = new KitList();
						}

						Global.VirtualDrums.tチップに対応するサウンド対応表を構築しサウンドインスタンスを生成する();
						//-----------------
						#endregion

						Global.Actフリップボード.t表示( "ユーザ「" + Global.Users.str現在のユーザ名 + "」でログインしました。" );
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ログアウト ]
				//-----------------
				case Eアプリ状態.ログアウト:
					{
						if( string.IsNullOrEmpty( Global.Users.str現在のユーザ名 ) )
							break;

						#region [ 仮想ドラムサウンドの解放 ]
						//-----------------
						C共通.tDisposeする( Global.VirtualDrums );
						//-----------------
						#endregion

						#region [ 曲リストの解放 ]
						//-----------------
						Global.Song.list曲リスト = new List<SongNode>();
						//-----------------
						#endregion

						#region [ BASS の解放…　は、次のユーザがログインするまで行わない。]
						//-----------------
						//C共通.tDisposeする( Global.Bass ); Global.Bass = null;
						//C共通.tDisposeする( ref Global.rc演奏用タイマ );
						//-----------------
						#endregion

						#region [ Theme の解放と初期化 ]
						//-----------------
						C共通.tDisposeする( Global.Theme );
						Global.Theme = new Theme();
						//-----------------
						#endregion

						#region [ Configファイルの保存と初期化 ]
						//-----------------
						Global.Users.Config.t保存する( Global.Folder.strConfigXMLの絶対パス );
						Global.Users.Config = new Config();
						//-----------------
						#endregion

						Trace.WriteLine( "" );
						Trace.WriteLine( string.Format( "ユーザ「{0}」をログアウトしました。", Global.Users.str現在のユーザ名 ) );
						Trace.WriteLine( "" );

						#region [ 現ユーザ用のTraceリスナがあれば削除する ]
						//-----------------
						for( int i = Trace.Listeners.Count - 1; i >= 1; i-- )	// 1つ（出力ウィンドウ）以上の追加リスナがあれば、すべて削除。
						{
							Trace.IndentLevel = 0;
							Trace.Listeners[ i ].Close();
							Trace.Listeners[ i ].Dispose();
							Trace.Listeners.RemoveAt( i );
						}
						//-----------------
						#endregion

						Global.Users.str現在のユーザ名 = null;
						Global.Folder.tユーザ個別フォルダを変更する( null );
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ユーザ別設定 ]
				//-----------------
				case Eアプリ状態.ユーザ別設定:
					{
						Global.tダイアログ表示準備();

						var dialog = new Cユーザ別設定ウィンドウ();
						dialog.Owner = Global.App.Window;
						Global.Input.Input管理.Dispose();	// メインフォームでのInput管理をいったん終了。

						// モーダルモードで起動する。閉じるまで処理が帰ってこない。
						// Input管理 はこの中で新しく作られ、設定ウィンドウ終了後に破棄される。
						dialog.tメイン処理( Global.App.D3D9Device );

						Global.App.Window.Select();
						Global.Input.Input管理 = new CInput管理( Global.App.Window.Handle );	// メインフォームでのInput管理を再開。

						Global.tダイアログ表示後始末();
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ D3Dデバイスの変更 ]
				//-----------------
				case Eアプリ状態.D3Dデバイスの変更:
					{
						if( this.bD3Dデバイスを変更する )
						{
							var newSettings = this.currentD3DSettings.Clone();
							newSettings.PresentParameters.Windowed = Global.Users.Config.Windowed;
							newSettings.PresentParameters.PresentationInterval = ( Global.Users.Config.VSyncWait ) ? PresentInterval.Default : PresentInterval.Immediate;
							newSettings.PresentParameters.BackBufferWidth = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Width : Global.Theme.FullscreenSize.Width;
							newSettings.PresentParameters.BackBufferHeight = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Height : Global.Theme.FullscreenSize.Height;

							this.tDirect3Dデバイスを生成・変更・リセットする(		// 例外はキャッチしない。準正常じゃなくて異常なので。
								newSettings,
								Global.Theme.LogicalDisplaySize,
								CApplicationForm.wsウィンドウスタイル,
								CApplicationForm.ws全画面スタイル,
								true );

							this.bD3Dデバイスを変更する = false;
						}
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ サウンドデバイスの変更 ]
				//-----------------
				case Eアプリ状態.サウンドデバイスの変更:
					{
						if( this.bサウンドデバイスを変更する )
						{
							Global.t現在のユーザConfigに従ってサウンドデバイスとすべての既存サウンドを再構築する();

							#region [ フリップボードに切替え結果を表示。 ]
							//-----------------
							switch( Global.SoundDevice.e出力デバイス )
							{
								case ESoundDeviceType.ASIO:
									Global.Actフリップボード.t表示( 
										string.Format( "ASIO デバイスに切替えました。[バッファ{0}ms(希望{1}ms)]",
											Global.SoundDevice.n実出力遅延ms, Global.Users.Config.SoundDelayASIO ) );
									break;

								case ESoundDeviceType.ExclusiveWASAPI:
									Global.Actフリップボード.t表示( 
										string.Format( "排他 WASAPI デバイスに切替えました。[バッファ{0}ms(希望{1}ms), 更新間隔{2:0.###}ms]",
											Global.SoundDevice.n実出力遅延ms, Global.Users.Config.SoundDelayExclusiveWASAPI, Global.Users.Config.SoundUpdatePeriodExclusiveWASAPI ) );
									break;

								case ESoundDeviceType.SharedWASAPI:
									Global.Actフリップボード.t表示( 
										string.Format( "共有 WASAPI デバイスに切替えました。[バッファ{0}ms, 更新間隔{2:0.###}ms]",
											Global.SoundDevice.n実出力遅延ms, Global.Users.Config.SoundDelaySharedWASAPI, Global.Users.Config.SoundUpdatePeriodSharedWASAPI ) );
									break;

								case ESoundDeviceType.DirectSound:
									Global.Actフリップボード.t表示( 
										string.Format( "DirectSound デバイスに切替えました。[バッファ{0}ms]", Global.SoundDevice.n実出力遅延ms ) );
									break;
							}
							//-----------------
							#endregion
						}
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ユーザ別曲管理 ]
				//-----------------
				case Eアプリ状態.ユーザ別曲管理:
					{
						Global.tダイアログ表示準備();

						var dialog = new Cユーザ別曲管理ウィンドウ();
						dialog.Owner = Global.App.Window;
						Global.Input.Input管理.Dispose();	// メインフォームでのInput管理をいったん終了。

						// モーダルモードで起動する。閉じるまで処理が帰ってこない。
						// Input管理 はこの中で新しく作られ、設定ウィンドウ終了後に破棄される。
						dialog.tメイン処理();

						Global.App.Window.Select();
						Global.Input.Input管理 = new CInput管理( Global.App.Window.Handle );	// メインフォームでのInput管理を再開。

						Global.tダイアログ表示後始末();

						// 曲リストをリセット。
						if( dialog.dialogResult != DialogResult.Cancel && Global.Song.list曲リスト.Count > 0 )
							Global.Stage.m選曲ステージ.t選択曲を最初の曲にリセットする();
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ 入力割り当て ]
				//-----------------
				case Eアプリ状態.入力割り当て:
					{
						Global.tダイアログ表示準備();

						var dialog = new C入力割当ウィンドウ();
						dialog.Owner = Global.App.Window;
						Global.Input.Input管理.Dispose();	// メインフォームでのInput管理をいったん終了。

						// モーダルモードで起動する。閉じるまで処理が帰ってこない。
						// Input管理 はこの中で新しく作られ、設定ウィンドウ終了後に破棄される。
						dialog.tメイン処理();

						Global.App.Window.Select();
						Global.Input.Input管理 = new CInput管理( Global.App.Window.Handle );	// メインフォームでのInput管理を再開。

						Global.tダイアログ表示後始末();

						// KeyAssign.ini を保存。
						Global.Input.KeyAssign.t保存する( Global.Folder.strKeyAssignXMLの絶対パス );
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ 曲読み込み ]
				//-----------------
				case Eアプリ状態.曲読み込み:
					{
						string path = ( Global.PlayerMode.bプレイヤーモードではない ) ? Global.Song.r現在演奏中の曲.ScoreFile : Global.PlayerMode.str演奏ファイル名;

						Global.Song.r現在演奏中の曲のスコア = Global.Song.tスコアを読み込む( path );

						if( Global.Song.r現在演奏中の曲のスコア != null )
						{
							// 読み込み成功
							this.e状態処理結果 = E状態処理結果.OK;
						}
						else
						{	
							// 読み込み失敗
							Global.Actフリップボード.t表示( "曲の読み込みに失敗しました。" );
							this.e状態処理結果 = E状態処理結果.NG;
						}
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ ウィンドウを前面へ ]
				//-----------------
				case Eアプリ状態.ウィンドウを前面へ:
					{
						Global.App.Window.Activate();
					}
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
				#region [ default ]
				//-----------------
				default:
					this.t完了( THREAD_GUI );
					break;
				//-----------------
				#endregion
			}
		}
		
		/// <summary>
		/// <para>進行処理を行う。</para>
		/// <para>ロックを得た進行スレッドにより実行される。</para>
		/// </summary>
		protected override void On進行()
		{
			switch( this.e現在の状態[ THREAD_進行 ] )
			{
				#region [ 待機 ]
				//-----------------
				case Eアプリ状態.待機:
					break;
				//-----------------
				#endregion
				#region [ アプリ終了 ]
				//-----------------
				case Eアプリ状態.アプリ終了:
					{
						this.bアプリケーションを終了する = true;
					}
					// 上記フラグを立てると、GUI・進行・描画スレッドは終了する。
					// これは CApplicationForm クラスの機能なので、スレッド終了時に ev状態処理完了通知[] が Set されているという保証はない。
					// そのため、全スレッドの通知を立てて、確実にフロー制御スレッドが終了するようにしておく。
					this.t全完了();
					break;
				//-----------------
				#endregion
				#region [ 選曲ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.選曲ステージ進行描画:
					this.t現在のステージの進行();
					break;
				//-----------------
				#endregion
				#region [ 演奏ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.演奏ステージ進行描画:
					this.t現在のステージの進行();
					break;
				//-----------------
				#endregion
				#region [ 結果ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.結果ステージ進行描画:
					this.t現在のステージの進行();
					break;
				//-----------------
				#endregion
				#region [ default ]
				//-----------------
				default:
					this.t完了( THREAD_進行 );
					break;
				//-----------------
				#endregion
			}
		}

		/// <summary>
		/// <para>描画処理を行う。</para>
		/// <para>ロックを得た描画スレッドにより実行される。</para>
		/// <para>BeginScene() と EndScene() の間に呼び出される。</para>
		/// <para>そのため、Direct3Dデバイスの変更を伴うような操作は行わないこと。</para>
		/// </summary>
		protected override void On描画()
		{
			switch( this.e現在の状態[ THREAD_描画 ] )
			{
				#region [ 待機 ]
				//-----------------
				case Eアプリ状態.待機:
					break;
				//-----------------
				#endregion
				#region [ ステージ以外のトップレベルActivityの活性化 ]
				//-----------------
				case Eアプリ状態.ステージ以外のトップレベルActivityの活性化:
					{
						this.tステージ以外のトップレベルActivityを活性化();
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 選曲ステージ活性化 ]
				//-----------------
				case Eアプリ状態.選曲ステージ活性化:
					{
						Global.Stage.m現在のステージ = Global.Stage.m選曲ステージ;
						Global.Stage.m選曲ステージ.On活性化( this.D3D9Device );
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 選曲ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.選曲ステージ進行描画:
					this.t現在のステージの描画();
					break;
				//-----------------
				#endregion
				#region [ 選曲ステージ非活性化 ]
				//-----------------
				case Eアプリ状態.選曲ステージ非活性化:
					{
						Global.Stage.m選曲ステージ.On非活性化();
						Global.Stage.m現在のステージ = null;
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ トップレベルActivityの非活性化 ]
				//-----------------
				case Eアプリ状態.トップレベルActivityの非活性化:
					{
						this.tすべてのトップレベルActivityを非活性化();
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 演奏ステージ活性化 ]
				//-----------------
				case Eアプリ状態.演奏ステージ活性化:
					{
						Global.Stage.m現在のステージ = Global.Stage.m演奏ステージ;
						Global.Stage.m演奏ステージ.On活性化( this.D3D9Device );
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 演奏ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.演奏ステージ進行描画:
					this.t現在のステージの描画();
					break;
				//-----------------
				#endregion
				#region [ 演奏ステージ非活性化 ]
				//-----------------
				case Eアプリ状態.演奏ステージ非活性化:
					{
						Global.Stage.m演奏ステージ.On非活性化();
						Global.Stage.m現在のステージ = null;
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 結果ステージ活性化 ]
				//-----------------
				case Eアプリ状態.結果ステージ活性化:
					{
						Global.Stage.m現在のステージ = Global.Stage.m結果ステージ;
						Global.Stage.m結果ステージ.On活性化( this.D3D9Device );
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 結果ステージ進行描画 ]
				//-----------------
				case Eアプリ状態.結果ステージ進行描画:
					this.t現在のステージの描画();
					break;
				//-----------------
				#endregion
				#region [ 結果ステージ非活性化 ]
				//-----------------
				case Eアプリ状態.結果ステージ非活性化:
					{
						Global.Stage.m結果ステージ.On非活性化();
						Global.Stage.m現在のステージ = null;
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ 曲の停止と解放 ]
				//-----------------
				case Eアプリ状態.曲の停止と解放:
					{
						if( Global.Song.r現在演奏中のスコアの背景動画 != null )
							Global.Song.r現在演奏中のスコアの背景動画.t再生停止();
						
						if( Global.Song.r現在演奏中のスコアのBGM != null )
							Global.Song.r現在演奏中のスコアのBGM.tサウンドを停止する();

						C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアの背景動画 );
						C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアのBGM );
						C共通.tDisposeする( ref Global.Song.r現在演奏中の曲のスコア );
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ プレイヤーモード時の演奏ステージの活性化 ]
				//-----------------
				case Eアプリ状態.プレイヤーモード時の演奏ステージの活性化:
					{
						// 演奏開始。
						
						Global.Users.Config.HitSound = Global.PlayerMode.b仮想ドラムあり;

						Global.Stage.m現在のステージ = Global.Stage.m演奏ステージ;
						Global.Stage.m演奏ステージ.On活性化( this.D3D9Device );		// スタンバイOK、演奏開始。
						// Global.Misc.Act背景動画.On活性化();	// 演奏ステージの活性化の中で Act背景動画 が活性化されるので、ここでは何もしない。


						// 指定された演奏位置へ移動。
						Global.Stage.m演奏ステージ.t演奏位置指定( Global.PlayerMode.n演奏開始小節番号 );
					}
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
				#region [ default ]
				//-----------------
				default:
					this.t完了( THREAD_描画 );
					break;
				//-----------------
				#endregion
			}
		}

		/// <summary>
		/// <para>アプリケーション全体のフローについて、現在の状態を管理し、状態に応じて各スレッドに指示を出す。</para>
		/// </summary>
		protected override void Onフロー制御()
		{
			// アプリの状態遷移図にそってフローが流れるようプログラミングする。
			// lock してないことに注意。

			try
			{
				// 起動

				this.t遷移( Eアプリ状態.アプリ起動 );
				this.t遷移( Eアプリ状態.ログイン名取得 );
				
				if( this.e状態処理結果 == E状態処理結果.CANCEL )
					goto Lアプリ終了;

			Lログイン:
				this.t遷移( Eアプリ状態.ログイン );			// this.str次のユーザ名 に新しいユーザ名が格納されていること。
				this.t遷移( Eアプリ状態.ステージ以外のトップレベルActivityの活性化 );
	
				if( Global.PlayerMode.bプレイヤーモードである )
				{
					Global.Song.r現在演奏中の曲のスコア = null;		// プレイヤーモード時はスコア = null のまま演奏ステージを活性化する。
					goto L演奏ステージ開始;
				}

				// 選曲ステージ

			L選曲ステージ開始:
				this.t遷移( Eアプリ状態.選曲ステージ活性化 );

			L選曲ステージ進行描画:
				this.t遷移( Eアプリ状態.選曲ステージ進行描画 );

				if( this.e状態処理結果 == E状態処理結果.曲決定 )
				{
					goto L曲読み込み;
				}
				else if( this.e状態処理結果 == E状態処理結果.終了 )
				{
					this.t遷移( Eアプリ状態.選曲ステージ非活性化 );
					goto Lログアウトして終了;
				}
				else if( this.e状態処理結果 == E状態処理結果.ユーザ別設定開始 )
				{
					this.t遷移( Eアプリ状態.ユーザ別設定 );
					this.t遷移( Eアプリ状態.D3Dデバイスの変更 );
					this.t遷移( Eアプリ状態.サウンドデバイスの変更 );
					goto L選曲ステージ進行描画;
				}
				else if( this.e状態処理結果 == E状態処理結果.ユーザ別曲管理開始 )
				{
					this.t遷移( Eアプリ状態.ユーザ別曲管理 );
					goto L選曲ステージ進行描画;
				}
				else if( this.e状態処理結果 == E状態処理結果.入力割り当て開始 )
				{
					this.t遷移( Eアプリ状態.入力割り当て );
					goto L選曲ステージ進行描画;
				}
				else if( this.e状態処理結果 == E状態処理結果.ログインユーザ変更 )
				{
					this.t遷移( Eアプリ状態.ログイン名取得 );

					if( this.e状態処理結果 != E状態処理結果.OK ||									// ユーザ選択をキャンセルされた
						this.str次のログインユーザ名.Equals( Global.Users.str現在のユーザ名 ) )		// 同一ユーザが選択された
						goto L選曲ステージ進行描画;

					this.t遷移( Eアプリ状態.ログアウト );
					this.t遷移( Eアプリ状態.トップレベルActivityの非活性化 );
					goto Lログイン;
				}

			L曲読み込み:
				this.t遷移( Eアプリ状態.曲読み込み );

				if( this.e状態処理結果 != E状態処理結果.OK )
				{
					Global.Stage.m選曲ステージ.t選曲し直す();	// volatile へアクセスするだけなのでロックなし。
					goto L選曲ステージ進行描画;
				}

				this.t遷移( Eアプリ状態.選曲ステージ非活性化 );

				// 演奏ステージ

			L演奏ステージ開始:
				this.t遷移( Eアプリ状態.演奏ステージ活性化 );
			
			L演奏ステージ進行描画:
				this.t遷移( Eアプリ状態.演奏ステージ進行描画 );

				if( this.e状態処理結果 == E状態処理結果.終了 )
				{
					this.t遷移( Eアプリ状態.曲の停止と解放 );
					this.t遷移( Eアプリ状態.演奏ステージ非活性化 );
					goto Lログアウトして終了;
				}
				else if( this.e状態処理結果 == E状態処理結果.CANCEL )
				{
					this.t遷移( Eアプリ状態.曲の停止と解放 );
					this.t遷移( Eアプリ状態.演奏ステージ非活性化 );
					C共通.t完全なガベージコレクションを実施する();
					goto L選曲ステージ開始;
				}
				else if( this.e状態処理結果 == E状態処理結果.GUI割込・演奏停止 )
				{
					this.t遷移( Eアプリ状態.曲の停止と解放 );
					this.t遷移( Eアプリ状態.演奏ステージ非活性化 );
					goto L演奏ステージ開始;		// 停止状態でスタンバイ。
				}
				else if( this.e状態処理結果 == E状態処理結果.GUI割込・演奏開始 )
				{
					this.t遷移( Eアプリ状態.曲の停止と解放 );
					this.t遷移( Eアプリ状態.演奏ステージ非活性化 );
					this.t遷移( Eアプリ状態.曲読み込み );

					if( this.e状態処理結果 == E状態処理結果.OK )
					{
						this.t遷移( Eアプリ状態.プレイヤーモード時の演奏ステージの活性化 );
						this.t遷移( Eアプリ状態.ウィンドウを前面へ );
					}
					else
					{
						this.t遷移( Eアプリ状態.演奏ステージ活性化 );
					}

					goto L演奏ステージ進行描画;	// 演奏開始
				}

				this.t遷移( Eアプリ状態.演奏ステージ非活性化 );

				// 結果ステージ

				this.t遷移( Eアプリ状態.結果ステージ活性化 );
				this.t遷移( Eアプリ状態.結果ステージ進行描画 );

				if( this.e状態処理結果 == E状態処理結果.終了 )
				{
					this.t遷移( Eアプリ状態.曲の停止と解放 );
					this.t遷移( Eアプリ状態.結果ステージ非活性化 );
					goto Lログアウトして終了;
				}

				this.t遷移( Eアプリ状態.曲の停止と解放 );
				this.t遷移( Eアプリ状態.結果ステージ非活性化 );
				goto L選曲ステージ開始;

				// アプリ終了
				
			Lログアウトして終了:
				this.t遷移( Eアプリ状態.ログアウト );
				this.t遷移( Eアプリ状態.トップレベルActivityの非活性化 );

			Lアプリ終了:
				this.t遷移( Eアプリ状態.アプリ終了 );
			}
			catch( FlowThreadAbortException )
			{
				return;	// スレッド終了。
			}
		}


		#region [| フロー制御用 |]
		//-----------------
		private enum Eアプリ状態
		{
			待機,
			アプリ起動,
			ログイン名取得,
			ログイン,
			ステージ以外のトップレベルActivityの活性化,
			ログアウト,
			トップレベルActivityの非活性化,
			アプリ終了,

			選曲ステージ活性化,
			選曲ステージ進行描画,
			選曲ステージ非活性化,
			ユーザ別設定,
			D3Dデバイスの変更,
			サウンドデバイスの変更,
			ユーザ別曲管理,
			入力割り当て,
			曲読み込み,

			演奏ステージ活性化,
			演奏ステージ進行描画,
			演奏ステージ非活性化,
			曲の停止と解放,

			結果ステージ活性化,
			結果ステージ進行描画,
			結果ステージ非活性化,

			プレイヤーモード時の演奏ステージの活性化,
			ウィンドウを前面へ,
		}
		public enum E状態処理結果 : int
		{
			OK,
			NG,
			CANCEL,
			ステージ継続,
			ユーザ別設定開始,
			ユーザ別曲管理開始,
			入力割り当て開始,
			ログインユーザ変更,
			曲決定,
			終了,
			CLEAR,
			//STAGEFAILED,
			GUI割込・演奏停止,
			GUI割込・演奏開始,
		}
		public volatile bool bWM_CLOSEを受け取った = false;

		private const int THREAD_GUI = 0;
		private const int THREAD_進行 = 1;
		private const int THREAD_描画 = 2;
		private const int THREAD_フロー制御 = 3;
		private volatile Eアプリ状態[] e現在の状態 = new Eアプリ状態[ 4 ] { Eアプリ状態.待機, Eアプリ状態.待機, Eアプリ状態.待機, Eアプリ状態.待機 };
		/// <summary>
		/// <para>状態処理完了時の結果コード。</para>
		/// <para>ev状態処理完了通知イベントを Set() する前に、このプロパティに結果を格納しておくこと。</para>
		/// </summary>
		private volatile E状態処理結果 e状態処理結果 = E状態処理結果.OK;
		/// <summary>
		/// <para>進行・描画・GUIスレッドの処理が完了したら、このイベントを Set() する。</para>
		/// </summary>
		private ManualResetEvent[] ev状態処理完了通知 = new ManualResetEvent[ 3 ] { 
			new ManualResetEvent( false ),	// [THREAD_GUI]
			new ManualResetEvent( false ),	// [THREAD_進行]
			new ManualResetEvent( false ),	// [THREAD_描画]
		};
		private class FlowThreadAbortException : Exception
		{
		}
		private ManualResetEvent[] ev全スレッドイベント = new ManualResetEvent[ 3 ];

		private void t遷移( Eアプリ状態 e次の状態 )
		{
			lock( this.obj排他用 )
			{
				if( this.bアプリケーションを終了する || this.bWindowClose済み )
					throw new FlowThreadAbortException();	// スレッド終了。

				Trace.TraceInformation( "フロー遷移：" + e次の状態 );

				for( int threadID = 0; threadID < 3; threadID++ )
				{
					this.e現在の状態[ threadID ] = e次の状態;		// 状態更新。完了時に「待機」に更新される。
					this.ev状態処理完了通知[ threadID ].Reset();
					this.ev全スレッドイベント[ threadID ] = this.ev状態処理完了通知[ threadID ];
				}
				this.e現在の状態[ THREAD_フロー制御 ] = e次の状態;	// 状態更新。ここでしか更新されない。
			}

			// 各スレッドの状態処理が全部終わるまでロック。

			Debug.WriteLine( e次の状態 + ": 待ち開始。" );
			ManualResetEvent.WaitAll( this.ev全スレッドイベント );
			Debug.WriteLine( e次の状態 + ": 待ち終了。" );
		}
		private void t完了( int threadID )
		{
			this.e現在の状態[ threadID ] = Eアプリ状態.待機;
			this.ev状態処理完了通知[ threadID ].Set();
		}
		private void t全完了()
		{
			this.t完了( THREAD_GUI );
			this.t完了( THREAD_進行 );
			this.t完了( THREAD_描画 );
		}
		//-----------------
		#endregion



		/// <summary>
		/// <para>終了処理を行う。</para>
		/// <para>フレームワークにより、メインループを抜けた後、一番最後に呼び出される。</para>
		/// <para>Direct3D, Device は Dispose 不要。</para>
		/// </summary>
		protected override void On終了()
		{
			Global.t終了();
			C共通.t完全なガベージコレクションを実施する();
		}

		/// <summary>
		/// <para>Managed リソースを生成する。</para>
		/// <para>フレームワークにより、デバイスを生成した直後に呼び出される。（リセットの直後には呼び出されない。）</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnManageリソースを生成する()
		{
			#region [ 黒タイルテクスチャの生成 ]
			//-----------------
			using( var bmp = new Bitmap( 256, 256 ) )
			using( var g = Graphics.FromImage( bmp ) )
			{
				g.FillRectangle( Brushes.Black, 0, 0, bmp.Width, bmp.Height );
				this.tx黒タイル = Global.tテクスチャを生成する( bmp );
			}
			//-----------------
			#endregion

			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnManagedリソースの作成( this.D3D9Device );
			}
		}

		/// <summary>
		/// <para>Managed リソースを解放する。</para>
		/// <para>デバイスを破棄する直前に呼び出される。（リセットの直前には呼び出されない。）</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnManageリソースを解放する()
		{
			#region [ 黒タイルテクスチャの破棄 ]
			//-----------------
			C共通.tDisposeする( ref this.tx黒タイル );
			//-----------------
			#endregion

			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnManagedリソースの解放();
			}
		}

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを生成する。</para>
		/// <para>フレームワークにより、デバイスロストした直後に呼び出される。</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnUnmanageリソースを生成する()
		{
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnUnmanagedリソースの作成( this.D3D9Device );
			}
		}

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを解放する。</para>
		/// <para>フレームワークにより、デバイスをリセットまたは破棄する直前に呼び出される。</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnUnmanageリソースを解放する()
		{
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnUnmanagedリソースの解放();
			}
		}


		#region [ protected ]
		//-----------------
		protected volatile List<CActivity> listトップレベルActivities = null;

		protected CTexture tx黒タイル = null;
		protected Rectangle rc黒タイルで塗りつぶす物理画面領域 = new Rectangle();
		protected void t描画範囲外をマスキングする()
		{
			// 論理画面以外の部分を黒テクスチャで覆い隠す。
			//
			//       　物理画面
			// ┌─┬──────┬─┐
			// │　│　　　上　　│　│		左図の上下左右の部分を
			// │　├──────┤　│		４回に分けて塗りつぶす。
			// │　│　論理画面　│　│
			// │左│（物理画面　│右│		領域は論理画面の座標で示すことに注意。
			// │　│　描画領域）│　│
			// │　├──────┤　│
			// │　│　　　下　　│　│
			// └─┴──────┴─┘

			// 以下、マネージドメモリの消費を抑えるため、new Rectangle() は使わない。

			// 左
			if( CTexture.rc物理画面描画領域.Left > 0 )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 右
			if( CTexture.rc物理画面描画領域.Right < CTexture.sz物理画面.Width )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Right;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz物理画面.Width - CTexture.rc物理画面描画領域.Right;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 上
			if( CTexture.rc物理画面描画領域.Top > 0 )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz論理画面.Width;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.rc物理画面描画領域.Top;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 下
			if( CTexture.rc物理画面描画領域.Bottom < CTexture.sz物理画面.Height )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = CTexture.rc物理画面描画領域.Bottom;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz論理画面.Width;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height - CTexture.rc物理画面描画領域.Bottom;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}
		}
		protected void t黒タイルテクスチャで領域を塗りつぶす( ref Rectangle rc塗りつぶす物理画面領域 )
		{
			// ※ 引数が ref なのは高速化のため。

			if( this.tx黒タイル == null )
				return;

			int nX方向枚数 = rc塗りつぶす物理画面領域.Width / this.tx黒タイル.sz画像サイズ.Width;
			int nX方向余り = rc塗りつぶす物理画面領域.Width % this.tx黒タイル.sz画像サイズ.Width;
			if( nX方向余り != 0 )
				nX方向枚数++;

			int nY方向枚数 = rc塗りつぶす物理画面領域.Height / this.tx黒タイル.sz画像サイズ.Height;
			int nY方向余り = rc塗りつぶす物理画面領域.Height % this.tx黒タイル.sz画像サイズ.Height;
			if( nY方向余り != 0 )
				nY方向枚数++;

			for( int y = 0; y < nY方向枚数; y++ )
			{
				var rc黒タイル = new Rectangle( 0, 0, this.tx黒タイル.sz画像サイズ.Width, this.tx黒タイル.sz画像サイズ.Height );

				if( ( nY方向余り != 0 ) && ( y == nY方向枚数 - 1 ) )
					rc黒タイル.Height = nY方向余り;		// 最後の1枚が余りの場合は縦サイズが違う

				for( int x = 0; x < nX方向枚数; x++ )
				{
					if( ( nX方向余り != 0 ) && ( x == nX方向枚数 - 1 ) )
						rc黒タイル.Width = nX方向余り;	// 最後の1枚が余りの場合は横サイズが違う

					tx黒タイル.t物理2D描画(		// 描画。
						this.D3D9Device,
						rc塗りつぶす物理画面領域.X + x * this.tx黒タイル.sz画像サイズ.Width,
						rc塗りつぶす物理画面領域.Y + y * this.tx黒タイル.sz画像サイズ.Height,
						rc黒タイル );
				}
			}
		}
		//-----------------
		#endregion

		#region [ イベント ]
		//-----------------
		protected void OnResizeEnd( object sender, EventArgs e )
		{
			lock( this.obj排他用 )
			{
				if( Global.Users.Config.Windowed == false )
					return;		// （たぶん来ないだろうが）全画面モードでは無視。

				Global.Users.Config.WindowClientSize =		// Config と
					Global.Theme.WindowClientSize =			// Theme の両方へ反映。
					this.Window.ClientSize;

				//Global.指示.b次のタイミングでD3Dデバイスを変更する = true;
			}
		}
		//-----------------
		#endregion

		#region [ private ]
		//-----------------
		private string str次のログインユーザ名 = null;

		private void t現在のステージの進行()
		{
			this.t入力をポーリングする();
			this.e状態処理結果 = (E状態処理結果) Global.Stage.m現在のステージ.On進行();
			this.tステージ以外のトップレベルActivityを進行();

			if( this.e状態処理結果 != E状態処理結果.ステージ継続 )
				this.t全完了();
		}
		private void t現在のステージの描画()
		{
			if( Global.Stage.m現在のステージ.b進行済み )
			{
				this.D3D9Device.BeginScene();

				Global.Stage.m現在のステージ.On描画( this.D3D9Device );
				this.tステージ以外のトップレベルActivityを描画( this.D3D9Device );
				this.t描画範囲外をマスキングする();

				this.D3D9Device.EndScene();
			}
		}
		private void t入力をポーリングする()
		{
			if( Global.Input != null &&
				Global.Input.Input管理 != null &&
				Global.Input.Input管理.hWnd == this.hWnd )	// 自分が Input管理 のオーナーであるときのみ更新。
			{
				Global.Input.t入力のポーリングとキー入力イベントへの変換( Global.bウィンドウがアクティブである, Global.Users.Config.BufferedInput );
			}
		}

		// トップレベルActivityを追加する際は、以下のメソッドをすべて修正すること。
		private void tトップレベルActivityを登録()
		{
			this.listトップレベルActivities = new List<CActivity>();

			this.listトップレベルActivities.Add( Global.Act英数字描画 );
			this.listトップレベルActivities.Add( Global.Actフリップボード );
			this.listトップレベルActivities.Add( Global.Stage.m演奏ステージ );
			this.listトップレベルActivities.Add( Global.Stage.m選曲ステージ );
			this.listトップレベルActivities.Add( Global.Stage.m結果ステージ );
		}
		private void tステージ以外のトップレベルActivityを活性化()
		{
			Global.Act英数字描画.On活性化( this.D3D9Device );
			Global.Actフリップボード.On活性化( this.D3D9Device );
		}
		private void tステージ以外のトップレベルActivityを進行()
		{
			Global.Actフリップボード.On進行();
		}
		private void tステージ以外のトップレベルActivityを描画( SlimDX.Direct3D9.Device device )
		{
			Global.Actフリップボード.On描画( device );
		}
		private void tすべてのトップレベルActivityを非活性化()
		{
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )		// ステージも込み
					act.On非活性化();
			}
		}
		//-----------------
		#endregion
	}
}
