﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using SlimDX;
using SlimDX.Direct3D9;
using SlimDX.Windows;

namespace FDK
{
	public class CApplicationForm : IDisposable
	{
		// プロパティ

		public RenderForm Window
		{
			get;
			protected set;
		}
		public Direct3D Direct3D
		{
			get;
			protected set;
		}
		public CD3DSettings currentD3DSettings
		{
			get;
			protected set;
		}
		public Device D3D9Device
		{
			get;
			protected set;
		}

		/// <summary>trueにすると、次のフレームタイミングでウィンドウを閉じ、Run() を抜ける。</summary>
		public bool bアプリケーションを終了する = false;

		public class Cウィンドウ状態
		{
			public CWin32.WINDOWPLACEMENT WindowPlacement;
			public bool bTopMostWindow;
		}
		public Cウィンドウ状態 mウインドウ状態 = null;

		public const uint wsウィンドウスタイル =
			CWin32.WS_OVERLAPPED |		// オーバラップウィンドウ。
			CWin32.WS_BORDER |			// 境界を持つ。
			CWin32.WS_CAPTION |			// タイトルバーを持つ。（WS_DLGFRAME スタイルと一緒に使うことは不可。）
			CWin32.WS_THICKFRAME |		// ウィンドウのサイズ変更に使うことができる太い枠を持つ。
			CWin32.WS_CLIPCHILDREN |	// 親ウィンドウの内部で描画するときに、子ウィンドウが占める領域を除外する。親ウィンドウを作成するときに使う。
			CWin32.WS_SYSMENU |			// タイトルバーにコントロールメニューボックスと「閉じる」ボタンを持つ。
			CWin32.WS_EX_APPWINDOW;		// ウィンドウが表示されているときには、必ずトップレベルウィンドウがタスクバー上に置かれる。

		public const uint ws全画面スタイル =
			CWin32.WS_POPUP |			// ポップアップウィンドウ。（タイトルバー、枠なし。クライアント領域のみ。）
			CWin32.WS_VISIBLE;			// 初期状態で可視。



		// コンストラクタ

		public CApplicationForm()
		{
			this.Window = null;
			this.Direct3D = null;
			this.D3D9Device = null;
			this.currentD3DSettings = null;
		}


		// メソッド

		/// <summary>
		/// <para>メインループを実行する。</para>
		/// <para>ウィンドウが閉じられるか例外が発生すると、このメソッドを抜ける。</para>
		/// </summary>
		public void Run()
		{
			#region [ Direct3D9 を生成する。]
			//-----------------
			try
			{
				this.Direct3D = new Direct3D();
			}
			catch
			{
				MessageBox.Show( "Direct3D9 の生成に失敗しました。\n終了します。", "StrokeStyle<T>エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
				return;
			}
			//-----------------
			#endregion

			#region [ アプリケーションを初期化し、this.Window と this.Direct3D9Device を生成する。]
			//-----------------
			try
			{
				this.On初期化();

				if( this.Window == null || this.D3D9Device == null )
					return;		// 終了。
			}
#if !DEBUG
			catch
			{
				MessageBox.Show( "初期化に失敗しました。\n終了します。", "StrokeStyle<T>エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
				return;
			}
#endif
			finally
			{
			}
			//-----------------
			#endregion

			#region [ ウィンドウイベントを設定する。]
			//-----------------
			this.Window.Closing += ( o, args ) => {
				this.bアプリケーションを終了する = true;
				this.bWindowClose済み = true;
			};
			//-----------------
			#endregion


			// アプリケーションのメインループに突入。

			MessagePump.Run( this.Window, () => {

				// 進行。

				this.On進行();


				// 終了チェック。

				if( this.bアプリケーションを終了する )
				{
					#region [ ウィンドウのクローズがまだなら実施する。（Alt+F4 の場合は既にクローズされている。）]
					//-----------------
					if( !this.bWindowClose済み )
					{
						this.Window.Close();
						this.bWindowClose済み = true;
					}
					//-----------------
					#endregion

					return;		// このデリゲートを終了。Run() からの帰還ではない。

					// Run() からの帰還は、ウィンドウのクローズが完了したときに行われる。
				}

	
				// 描画。

				#region [ デバイスを消失してたら復元を試みる。]
				//-----------------
				if( this.bデバイス消失 )
				{
					if( this.Window.WindowState == FormWindowState.Minimized )
					{
						Trace.TraceWarning( "ウィンドウが最小化されています。描画を500ms待機します。" );
						Thread.Sleep( 500 );
					}
					else
					{
						// Direct3Dデバイスの復元

						if( this.D3D9Device.TestCooperativeLevel() == ResultCode.DeviceNotReset )
						{
							// 新しいD3D設定を作成。

							var newSettings = this.currentD3DSettings.Clone();
							if( this.currentD3DSettings.PresentParameters.Windowed )
							{
								#region [ ウィンドウモードの場合、BackBuffer フォーマットと Desktop フォーマットを一致させる。]
								//-----------------
								DisplayMode dm = this.Direct3D.GetAdapterDisplayMode( this.currentD3DSettings.nAdaptor );
								if( this.currentD3DSettings.PresentParameters.BackBufferFormat != dm.Format )
								{
									// BackBuffer フォーマットが Desktopフォーマットと異なるなら後者に合わせる。
									newSettings.PresentParameters.BackBufferFormat = dm.Format;
								}
								//-----------------
								#endregion
							}

							try
							{
								if( this.tDirect3Dデバイスを生成・変更・リセットする( newSettings ) )
								{
									Trace.TraceInformation( "Direct3Dデバイスを復元しました。" );
									this.bデバイス消失 = false;
								}
							}
							catch
							{
								Trace.TraceInformation( "復元に失敗しました。500ms後に再度試みます。" );
								Thread.Sleep( 500 );
							}
						}
						else
						{
							Trace.TraceInformation( "まだ復元できない状態です。500ms 後に再度試みます。" );
							Thread.Sleep( 500 );
						}
					}
				}
				//-----------------
				#endregion

				try
				{
					this.D3D9Device.BeginScene();
					this.On描画();
					this.D3D9Device.EndScene();
					this.D3D9Device.Present();
				}
				catch( Direct3D9Exception e )	// デバイス関連以外の例外はそのまま発出する。
				{
					#region [ DeviceLost 以外の例外が発生したらアプリを終了する。]
					//-----------------
					if( e.ResultCode == ResultCode.DeviceLost )
					{
						Trace.TraceInformation( "Direct3Dデバイスを消失しました。" );
						this.bデバイス消失 = true;
					}
					else
					{
						Trace.TraceInformation( "Direct3Dデバイスにおいて、DeviceLost 以外の例外が発生しました。" );
						throw;
					}
					//-----------------
					#endregion
				}

			} );


			// アプリ正常終了。

			this.OnManageリソースを解放する();
			this.OnUnmanageリソースを解放する();
			this.On終了();
		}

		/// <summary>
		/// <para>Direct3Dデバイスの生成、変更、リセットを行う。</para>
		/// <para>新しい設定と現在の設定とを比較し、生成、変更、リセットのいずれかを実行する。</para>
		/// <para>ウィンドウのクライアントサイズはバックバッファに等しく設定される。</para>
		/// <para>処理に成功すれば true を返す。処理に失敗すれば、準正常系は false を返し、異常系は例外を発出する。</para>
		/// </summary>
		public bool tDirect3Dデバイスを生成・変更・リセットする( CD3DSettings newD3DSettings, Size sz論理画面, uint wsウィンドウモード時のウィンドウスタイル, uint ws全画面モード時のウィンドウスタイル, bool bマウスカーソルの表示を制御する )
		{
			if( this.Window == null )
				throw new InvalidOperationException( "ウィンドウが未生成のままDirect3D9デバイスを生成しようとしました。" );

			bool b初めての生成 = ( this.currentD3DSettings == null );
			bool bウィンドウモードへ移行 = ( newD3DSettings.PresentParameters.Windowed );
			bool b全画面モードへ移行 = !bウィンドウモードへ移行;
			bool b全画面からウィンドウへの切替え =
				this.currentD3DSettings != null &&
				this.currentD3DSettings.PresentParameters.Windowed == false &&
				newD3DSettings.PresentParameters.Windowed == true;
			bool bウィンドウから全画面への切替え =
				this.currentD3DSettings != null &&
				this.currentD3DSettings.PresentParameters.Windowed == true &&
				newD3DSettings.PresentParameters.Windowed == false;


			var oldD3DSettings = ( b初めての生成 ) ? null : this.currentD3DSettings.Clone();

			#region [ ウィンドウスタイルの設定。]
			//-----------------
			if( b全画面からウィンドウへの切替え || ( b初めての生成 && bウィンドウモードへ移行 ) )
			{
				#region [ ウィンドウモード用のウィンドウスタイルを設定する。]
				//-----------------
				CWin32.SetWindowLong( this.Window.Handle, CWin32.GWL_STYLE, CApplicationForm.wsウィンドウスタイル );
				//-----------------
				#endregion
			}
			else if( bウィンドウから全画面への切替え || ( b初めての生成 && b全画面モードへ移行 ) )
			{
				#region [ 現在のウィンドウパラメータを保存する。]
				//-----------------
				this.mウインドウ状態 = new Cウィンドウ状態() {
					bTopMostWindow = this.Window.TopMost,
					WindowPlacement = new CWin32.WINDOWPLACEMENT() { length = CWin32.WINDOWPLACEMENT.Length },
				};
				CWin32.GetWindowPlacement( this.Window.Handle, ref this.mウインドウ状態.WindowPlacement );
				//-----------------
				#endregion
				#region [ 全画面用のウィンドウスタイルを設定する。]
				//-----------------
				CWin32.SetWindowLong( this.Window.Handle, CWin32.GWL_STYLE, CApplicationForm.ws全画面スタイル );
				//-----------------
				#endregion
			}
			//-----------------
			#endregion
			#region [ ウィンドウのクライアントサイズをバックバッファに合わせる。]
			//-----------------
			if( this.Window.ClientSize.Width != newD3DSettings.PresentParameters.BackBufferWidth ||
				this.Window.ClientSize.Height != newD3DSettings.PresentParameters.BackBufferHeight )
			{
				this.Window.ClientSize = new Size( newD3DSettings.PresentParameters.BackBufferWidth, newD3DSettings.PresentParameters.BackBufferHeight );
			}
			//-----------------
			#endregion

			#region [ Direct3D9デバイスをリセットまたは新規生成する。]
			//-----------------
			bool bリセットだけでOK = ( !b初めての生成 && oldD3DSettings.bデバイスの再生成が不要でリセットだけで済む( newD3DSettings ) );

			if( bリセットだけでOK )
			{
				#region [ デバイスのリセット ]
				//-----------------
				this.OnUnmanageリソースを解放する();
				if( this.D3D9Device.Reset( newD3DSettings.PresentParameters ) != ResultCode.DeviceLost )
				{
					Trace.TraceInformation( "デバイスをリセットしました。" );
					this.OnUnmanageリソースを生成する();
					bリセットだけでOK = true;
				}
				else
				{
					Trace.TraceWarning( "リセットに失敗しました。続けて、デバイスの新規生成を行います。" );
					bリセットだけでOK = false;
				}
				//-----------------
				#endregion
			}
			if( !bリセットだけでOK ) // ← "else if" にしないこと。リセットに失敗した場合、bリセットでOK = false に変えてここに来るため。
			{
				#region [ デバイスの新規生成 ]
				//-----------------
				this.OnManageリソースを解放する();
				this.OnUnmanageリソースを解放する();
				C共通.tDisposeする( this.D3D9Device );

				this.D3D9Device = new Device(		// 失敗したら異常系とみなし、そのまま例外をthrow。
					this.Direct3D,
					newD3DSettings.nAdaptor,
					newD3DSettings.DeviceType,
					newD3DSettings.PresentParameters.DeviceWindowHandle,
					newD3DSettings.CreateFlags,
					newD3DSettings.PresentParameters );

				Trace.TraceInformation( "デバイスを生成しました。" );
				this.D3D9Device.SetDialogBoxMode( true );

				this.OnManageリソースを生成する();
				this.OnUnmanageリソースを生成する();
				//-----------------
				#endregion
			}
			//-----------------
			#endregion

			this.currentD3DSettings = newD3DSettings.Clone();


			this.OnD3Dデバイスステータスの初期化();

			#region [ ウィンドウパラメータを復元する（全画面→ウィンドウ切替えのときのみ）]
			//-----------------
			if( b全画面からウィンドウへの切替え )
			{
				if( this.mウインドウ状態 != null )	// 全画面で起動していま初めてウィンドウモードにする場合は、復元するものがない。
				{
					CWin32.SetWindowPlacement( this.Window.Handle, ref this.mウインドウ状態.WindowPlacement );
					this.Window.TopMost = this.mウインドウ状態.bTopMostWindow;
				}
			}
			//-----------------
			#endregion
			#region [ ウィンドウを不可視にしてたら可視に戻す。]
			//-----------------
			if( !this.Window.Visible )
				CWin32.ShowWindow( this.Window.Handle, CWin32.EShowWindow.Show );
			//-----------------
			#endregion
			
			#region [ 長期入力無反応時のアクションの設定。]
			//-----------------
			if( newD3DSettings.PresentParameters.Windowed )
			{
				// ウィンドウモード：システムは、入力反応が無ければスリープしたりモニタをOFFにしたりしてよい。
				CWin32.SetThreadExecutionState( CWin32.ES_SYSTEM_REQUIRED | CWin32.ES_CONTINUOUS );
			}
			else
			{
				// 全画面モード：システムは、入力反応が無くてもスリープしたりモニタをいじったりしてはいけない。
				CWin32.SetThreadExecutionState( CWin32.ES_SYSTEM_REQUIRED | CWin32.ES_DISPLAY_REQUIRED | CWin32.ES_CONTINUOUS );
			}
			//-----------------
			#endregion
			
			#region [ マウスカーソルの表示・非表示 ]
			//-----------------
			if( bマウスカーソルの表示を制御する )
			{
				if( bウィンドウから全画面への切替え || ( b初めての生成 && b全画面モードへ移行 ) )
					this.tマウスカーソルを消す();

				if( b全画面からウィンドウへの切替え || ( b初めての生成 && bウィンドウモードへ移行 ) )
					this.tマウスカーソルを表示する();
			}
			//-----------------
			#endregion

			#region [ CTexture の画面情報の更新 ]
			//-----------------
			if( !sz論理画面.IsEmpty ) CTexture.sz論理画面 = sz論理画面;
			CTexture.sz物理画面 = this.Window.ClientSize;
			CTexture.rc物理画面描画領域 = new Rectangle();

			Vector2 vc論理画面を1とする場合の物理画面の倍率 = new Vector2() {
				X = (float) CTexture.sz物理画面.Width / CTexture.sz論理画面.Width,
				Y = (float) CTexture.sz物理画面.Height / CTexture.sz論理画面.Height,
			};

			if( vc論理画面を1とする場合の物理画面の倍率.X < vc論理画面を1とする場合の物理画面の倍率.Y )
			{
				// (A) 物理と論理の横幅を一致させる。

				CTexture.f画面比率 = vc論理画面を1とする場合の物理画面の倍率.X;
				CTexture.rc物理画面描画領域.Width = CTexture.sz物理画面.Width;
				CTexture.rc物理画面描画領域.Height = (int) ( CTexture.sz論理画面.Height * CTexture.f画面比率 );
				CTexture.rc物理画面描画領域.X = 0;
				CTexture.rc物理画面描画領域.Y = ( CTexture.sz物理画面.Height - CTexture.rc物理画面描画領域.Height ) / 2;
			}
			else
			{
				// (B) 物理と論理の縦幅を一致させる。

				CTexture.f画面比率 = vc論理画面を1とする場合の物理画面の倍率.Y;
				CTexture.rc物理画面描画領域.Width = (int) ( CTexture.sz論理画面.Width * CTexture.f画面比率 );
				CTexture.rc物理画面描画領域.Height = CTexture.sz物理画面.Height;
				CTexture.rc物理画面描画領域.X = ( CTexture.sz物理画面.Width - CTexture.rc物理画面描画領域.Width ) / 2;
				CTexture.rc物理画面描画領域.Y = 0;
			}
			//-----------------
			#endregion

			return true;
		}
		public bool tDirect3Dデバイスを生成・変更・リセットする( CD3DSettings newD3DSettings, Size sz論理画面, uint wsウィンドウモード時のウィンドウスタイル, uint ws全画面モード時のウィンドウスタイル )
		{
			return this.tDirect3Dデバイスを生成・変更・リセットする( newD3DSettings, sz論理画面, wsウィンドウモード時のウィンドウスタイル, ws全画面モード時のウィンドウスタイル, true );
		}
		public bool tDirect3Dデバイスを生成・変更・リセットする( CD3DSettings newD3DSettings, Size sz論理画面 )
		{
			return this.tDirect3Dデバイスを生成・変更・リセットする( newD3DSettings, sz論理画面, uint.MaxValue, uint.MaxValue, true );
		}
		public bool tDirect3Dデバイスを生成・変更・リセットする( CD3DSettings newD3DSettings )
		{
			return this.tDirect3Dデバイスを生成・変更・リセットする( newD3DSettings, Size.Empty, uint.MaxValue, uint.MaxValue, true );
		}

		public void tデバイスをクリアする()
		{
			this.D3D9Device.Clear( ClearFlags.Target, this.colorデバイスクリア色, 0.0f, 0 );
		}

		public void tマウスカーソルを表示する()
		{
			if( !this.bマウスカーソル表示中 )
			{
				Cursor.Show();
				this.bマウスカーソル表示中 = true;
			}
		}
		public void tマウスカーソルを消す()
		{
			if( this.bマウスカーソル表示中 )
			{
				Cursor.Hide();
				this.bマウスカーソル表示中 = false;
			}
		}

	
		// 子クラスで override するメソッド群

		/// <summary>
		/// <para>this.Window の生成、this.Direct3DDevice の生成を行う。</para>
		/// <para>this.Direct3D の生成の後に呼び出される。</para>
		/// <para>this.Direct3DDevice は、this.Window を生成した後に、CApplicationForm.tDirect3Dデバイスの生成または変更( CD3DSettings newSettings ) を用いて生成すること。</para>
		/// <para>エラー等でアプリを終了したい場合は例外を発生させること。</para>
		/// </summary>
		protected virtual void On初期化() { }

		/// <summary>
		/// <para>Direct3Dデバイスに対する設定を行う。</para>
		/// <para>デバイスのリセット時や再作成時に呼び出される。</para>
		/// </summary>
		protected virtual void OnD3Dデバイスステータスの初期化()
		{
			// this.Device.SetTransform()
			// this.Device.SetRenderState()
			// this.Device.SetTextureStageState() 
			// etc,.
		}

		/// <summary>
		/// <para>進行処理を行う。</para>
		/// <para>BeginScene～EndScene の間に行えない処理もここで行う。</para>
		/// </summary>
		protected virtual void On進行() { }

		/// <summary>
		/// <para>描画処理を行う。</para>
		/// <para>BeginScene() と EndScene() の間に呼び出される。</para>
		/// <para>そのため、Direct3Dデバイスを変更するような操作は行わないこと。</para>
		/// </summary>
		protected virtual void On描画() { }

		/// <summary>
		/// <para>終了処理を行う。</para>
		/// <para>メインループを抜けた後、一番最後に呼び出される。</para>
		/// <para>Direct3D, Device は Dispose 不要。</para>
		/// </summary>
		protected virtual void On終了() { }

		/// <summary>
		/// <para>Managed リソースを生成する。</para>
		/// <para>デバイスを生成した直後に呼び出される。（リセットの直後には呼び出されない。）</para>
		/// </summary>
		protected virtual void OnManageリソースを生成する() { }

		/// <summary>
		/// <para>Managed リソースを解放する。</para>
		/// <para>デバイスを破棄する直前に呼び出される。（リセットの直前には呼び出されない。）</para>
		/// </summary>
		protected virtual void OnManageリソースを解放する() { }

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを生成する。</para>
		/// <para>デバイスロストした直後に呼び出される。</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected virtual void OnUnmanageリソースを生成する() { }

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを解放する。</para>
		/// <para>デバイスをリセットまたは破棄する直前に呼び出される。</para>
		/// </summary>
		protected virtual void OnUnmanageリソースを解放する() { }

		#region [ IDisposable 実装 ]
		//-----------------
		public void Dispose()
		{
			C共通.tDisposeする( this.D3D9Device ); this.D3D9Device = null;
			C共通.tDisposeする( this.Direct3D ); this.Direct3D = null;
		}
		//-----------------
		#endregion

		#region [ protected ]
		//-----------------
		protected bool bデバイス消失 = false;
		protected bool bWindowClose済み = false;
		protected bool bマウスカーソル表示中 = true;
		protected readonly Color4 colorデバイスクリア色 = new Color4( 1f, 0f, 0f, 0f );
		//-----------------
		#endregion
	}
}
