﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;

namespace FDK
{
	/// <summary>
	/// <para>CDirectShow にて取得されるスナップイメージを表す。</para>
	/// <para>通常は、取得後に tビットマップに変換して返す() を呼び出し、System.Drawing.Bitmap に変換して活用する。</para>
	/// </summary>
	public class CDirectShowSnapImage : IDisposable
	{
		// プロパティ

		public int n幅px
		{
			get;
			protected set;
		}
		public int n高さpx
		{
			get;
			protected set;
		}
		public int nスキャンライン幅byte
		{
			get;
			protected set;
		}
		public int nデータサイズbyte
		{
			get;
			protected set;
		}
		public IntPtr ipイメージデータ
		{
			get;
			protected set;
		}


		// メソッド

		public CDirectShowSnapImage( IntPtr ipイメージデータ, int n幅px, int n高さpx, int nスキャンライン幅byte, int nデータサイズbyte )
		{
			this.ipイメージデータ = ipイメージデータ;
			this.n幅px = n幅px;
			this.n高さpx = n高さpx;
			this.nスキャンライン幅byte = nスキャンライン幅byte;
			this.nデータサイズbyte = nデータサイズbyte;
		}

		/// <summary>
		/// <para>イメージデータからビットマップを生成して返す。</para>
		/// <para>このインスタンス（CDirectShowSnapImage）は、Bitmap が利用中は Dispose してはいけない？（詳細不明）</para>
		/// </summary>
		public Bitmap tビットマップに変換して返す()
		{
			int iBufSize = this.n幅px * this.n高さpx * 4;	// サンプルグラバのメディアタイプで RGB32 を指定しているものと想定とし、1ピクセルは4バイトと勝手にみなす。

			return new Bitmap(
				this.n幅px,
				this.n高さpx,
				-this.nスキャンライン幅byte,
				PixelFormat.Format32bppRgb,
				(IntPtr) ( this.ipイメージデータ.ToInt32() + iBufSize - this.nスキャンライン幅byte )
			);
		}

		#region [ Dispose-Finalize パターン実装 ]
		//-----------------
		public void Dispose()
		{
			this.Dispose( true );
			GC.SuppressFinalize( this );	// ちゃんと Dispose されたので、ファイナライズ不要であることを CLR に伝える。
		}
		protected virtual void Dispose( bool bManagedリソースも解放する )
		{
			if( bManagedリソースも解放する )
			{
				// Managed リソースの解放
			}

			// Unmanaged リソースの解放

			if( this.ipイメージデータ != null )
			{
				Marshal.FreeCoTaskMem( this.ipイメージデータ );
				this.ipイメージデータ = IntPtr.Zero;
			}
		}
        ~CDirectShowSnapImage()
		{
			// ファイナライザが呼ばれたということは、Dispose() されなかったということ。
			// この場合、Managed リソースは先にファイナライズされている可能性があるので、Unmamaed リソースのみを解放する。
			
			this.Dispose( false );
        }
		//-----------------
		#endregion
	}
}
