﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Security;
using SlimDX;
using SlimDX.Direct3D9;
using SlimDX.Windows;
using SlimDX.DirectInput;
using FDK;
using DirectShowLib;

namespace StrokeStyleT
{
	class App : CApplicationForm
	{
		public void tユーザのログイン処理( string str新しいユーザ名 )
		{
			// ログイン(1) Config読み込み と ログリスナの追加

			Global.Users.str現在のユーザ名 = str新しいユーザ名;
			Global.Folder.tユーザを変更する( str新しいユーザ名 );

			#region[ 新ユーザの Config を読み込む。]
			//-----------------
			Global.Users.Config = Config.t読み込む( Global.Folder.strConfigXMLの絶対パス );

			// プレイヤーモードなら AutoPlay をすべて ON にする。
			// （プレイヤーモードではConfigを保存しないので、好きに変更しても大丈夫。）

			if( Global.PlayerMode.bプレイヤーモードである )
				Global.Users.Config.tすべてのAutoPlayを設定する( true );
			//-----------------
			#endregion

			#region [ Traceリスナを追加する。]
			//-----------------
			Trace.AutoFlush = true;
			Trace.IndentLevel = 0;

			if( Global.Users.Config.WriteLog )	// ログ出力する場合のみ
			{
				string path = Path.Combine( Global.Folder.strユーザ個別フォルダ, Properties.Resources.LOG_FILE_NAME );

				try
				{
					Trace.Listeners.Add(
						new CTraceLogListener(
							new StreamWriter( path, false, Encoding.GetEncoding( "shift-jis" ) ) ) );
				}
				catch( Exception e )
				{
					#region [ 全画面モードならマウスカーソルを表示する。]
					//-----------------
					if( !this.currentD3DSettings.PresentParameters.Windowed )
						this.tマウスカーソルを表示する();
					//-----------------
					#endregion

					MessageBox.Show( string.Format( "{0} へのリスナ追加に失敗しました。\nログファイルは出力または上書きされません。\n---------\n{1}", path, e.ToString() ), "エラー", MessageBoxButtons.OK );

					#region [ 全画面モードならマウスカーソルを消す。]
					//-----------------
					if( !this.currentD3DSettings.PresentParameters.Windowed )
						this.tマウスカーソルを消す();
					//-----------------
					#endregion

					//throw;	終了しない。
				}
			}
			//-----------------
			#endregion
			
			// 以下、ログファイル出力可能。


			#region [ ログにタイトルを表示する。]
			//-----------------
			Trace.WriteLine( "" );
			Trace.Write( ( Global.PlayerMode.bプレイヤーモードである ) ? Properties.Resources.TITLE_PLAYERMODE : Properties.Resources.TITLE_NORMALMODE );
			Trace.WriteLine( string.Format( " {0}.{1} (rev.{2} build.{3})", Global.nメジャー番号, Global.nマイナー番号, Global.nリビジョン番号, Global.nビルド番号 ) );
			Trace.WriteLine( "Copyright 2009 2011 (C) ProjectSST All Rights Reserved." );
			Trace.WriteLine( "" );
			Trace.WriteLine( string.Format( "ユーザ「{0}」でログインします。", str新しいユーザ名 ) );
			Trace.WriteLine( "" );
			//-----------------
			#endregion


			// ログイン(2) その他の処理

			#region [ ウィンドウが未生成（＝最初のログイン）なら生成する。]
			//-----------------
			if( this.Window == null )
			{
				this.Window = new Cメインウィンドウ( this ) {
					Text = ( Global.PlayerMode.bプレイヤーモードである ) ? 
						Properties.Resources.TITLE_PLAYERMODE :
						string.Format( "{0} {1}.{2} (rev.{3} build.{4})", 
							Properties.Resources.TITLE_NORMALMODE, 
							Global.nメジャー番号,
							Global.nマイナー番号,
							Global.nリビジョン番号, 
							Global.nビルド番号 ),
					ClientSize = Global.Theme.WindowClientSize,
					MinimizeBox = false,
					MaximizeBox = false,
				};
				this.Window.Activated += ( o, args ) => { Global.bウィンドウがアクティブである = true; };
				this.Window.Deactivate += ( o, args ) => { Global.bウィンドウがアクティブである = false; };
				this.Window.ResizeEnd += new EventHandler( this.OnResizeEnd );
			}
			//-----------------
			#endregion

			Global.t現在のユーザConfigに従ってサウンドを構築する();		// DirectSound が hWnd を必要とするので、ウィンドウ生成後に呼び出すこと。

			Global.Stage.m演奏ステージ.t演奏レーンチップ対応表をConfigに合わせて再構築する();

			#region[ Theme の構築（サウンドデバイス生成後に呼び出すこと。） ]
			//-----------------
			if( Global.Theme != null )
				C共通.tDisposeする( Global.Theme );	// 古いThemeがあれば解放する。（最初のログイン時）

			Global.Theme = Theme.t読み込む( Global.Theme.tファイルパス( Properties.Resources.XMLNAME_THEME ) );
			Global.Theme.t初期化( this.Direct3D );

			if( Global.PlayerMode.bプレイヤーモードである )		// プレイヤーモード時においては強制的にウィンドウモードとする。
				Global.Users.Config.Windowed = true;

			// Config からの反映

			if( Global.Users.Config.WindowClientSize.Width > 0 ||
				Global.Users.Config.WindowClientSize.Height > 0 )
			{
				Global.Theme.WindowClientSize = Global.Users.Config.WindowClientSize;
			}
			if( Global.Users.Config.FullscreenSize.Width > 0 ||
				Global.Users.Config.FullscreenSize.Height > 0 )
			{
				Global.Theme.FullscreenSize = Global.Users.Config.FullscreenSize;
			}
			if( Global.Users.Config.FullscreenRefreshRate > 0 )
				Global.Theme.FullscreenRefreshRate = Global.Users.Config.FullscreenRefreshRate;

			//-----------------
			#endregion

			#region [ Direct3Dデバイスのリセットまたは新規生成 ]
			//-----------------
			this.tDirect3Dデバイスを生成・変更・リセットする(
				new CD3DSettings() {
					nAdaptor = 0,
					DeviceType = SlimDX.Direct3D9.DeviceType.Hardware,
					CreateFlags = CreateFlags.HardwareVertexProcessing,
					PresentParameters = new PresentParameters() {
						BackBufferCount = 1,
						BackBufferWidth = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Width : Global.Theme.FullscreenSize.Width,
						BackBufferHeight = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Height : Global.Theme.FullscreenSize.Height,
						Windowed = Global.Users.Config.Windowed,
						BackBufferFormat = Global.Theme.BackbufferFormat,	// GUI を使う場合、GDIと互換性のあるフォーマットでなければならない。
						DeviceWindowHandle = this.Window.Handle,
						Multisample = MultisampleType.None,					// GDI を使う場合、None でなければならない。
						SwapEffect = SwapEffect.Discard,					// GUI を使う場合、Discard でなければならない。
						PresentationInterval = ( Global.Users.Config.VSyncWait ) ? PresentInterval.Default : PresentInterval.Immediate,
						PresentFlags = PresentFlags.LockableBackBuffer,		// GDI を使う場合、LockableBackBuffer でなければならない。
					}
				},
				Global.Theme.LogicalDisplaySize,
				CApplicationForm.wsウィンドウスタイル,
				CApplicationForm.ws全画面スタイル,
				true );
			//-----------------
			#endregion

			#region[ 曲リストの読み込みと構築 ]
			//-----------------
			if( Global.PlayerMode.bプレイヤーモードではない )
			{
				Global.Song.t曲リストをクリアしConfigに記されているSongListをすべて読み込んでマージする();
				Global.Stage.m選曲ステージ.t選択曲を最初の曲にリセットする();
			}
			else
				Global.Song.list曲リスト = new List<SongNode>();	// プレイヤーモードでは曲リストを構築しない。
			//-----------------
			#endregion

			#region [ 仮想ドラムサウンドの構築 ]
			//-----------------
			if( Global.Users.Config.KitListFiles.Count > 0 )
			{
				// (A) KitListFiles 要素があった場合 → 先頭にあるキットを選択。今のところ複数キット対応は未実装。
				string fileName = Global.Users.Config.KitListFiles[ 0 ];
				Global.VirtualDrums.KitList = KitList.t読み込む( fileName );
			}
			else
			{
				// (B) KitListFiles 要素がなかった場合 → 規定のドラムを使用。
				Global.VirtualDrums.KitList = new KitList();
			}

			Global.VirtualDrums.tチップに対応するサウンド対応表を構築しサウンドインスタンスを生成する();
			//-----------------
			#endregion

			Global.Actフリップボード.t表示( "ユーザ「" + str新しいユーザ名 + "」でログインしました。" );
		}
		public void tユーザのログアウト処理()
		{
			#region [ 仮想ドラムサウンドの解放 ]
			//-----------------
			C共通.tDisposeする( Global.VirtualDrums );
			//-----------------
			#endregion
	
			#region [ 曲リストの解放 ]
			//-----------------
			Global.Song.list曲リスト = new List<SongNode>();
			//-----------------
			#endregion

			#region [ BASS の解放…　は、次のユーザがログインするまで行わない。]
			//-----------------
			//C共通.tDisposeする( Global.Bass ); Global.Bass = null;
			//C共通.tDisposeする( ref Global.rc演奏用タイマ );
			//-----------------
			#endregion

			#region [ Theme の解放と初期化 ]
			//-----------------
			C共通.tDisposeする( Global.Theme );
			Global.Theme = new Theme();
			//-----------------
			#endregion

			#region [ Configファイルの保存と初期化 ]
			//-----------------
			Global.Users.Config.t保存する( Global.Folder.strConfigXMLの絶対パス );
			Global.Users.Config = new Config();
			//-----------------
			#endregion

			#region [ 現ユーザ用のTraceリスナがあれば削除する ]
			//-----------------
			for( int i = Trace.Listeners.Count - 1; i >= 1; i-- )	// 1つ（出力ウィンドウ）以上の追加リスナがあれば、すべて削除。
			{
				Trace.IndentLevel = 0;
				Trace.Listeners[ i ].Close();
				Trace.Listeners[ i ].Dispose();
				Trace.Listeners.RemoveAt( i );
			}
			//-----------------
			#endregion

			Global.Users.str現在のユーザ名 = null;
			Global.Folder.tユーザを変更する( null );
		}

		public App()
		{
			Global.t初期化( this );		// Program.Main() から参照されるので、先にこれだけ初期化。
		}

		#region [ オーバーライドメソッド ]
		//-----------------
		/// <summary>
		/// <para>this.Window の生成、this.Direct3DDevice の生成を行う。</para>
		/// <para>this.Direct3D の生成の後に呼び出される。</para>
		/// <para>this.Direct3DDevice は、this.Window を生成した後に、CApplicationForm.tDirect3Dデバイスの生成または変更( CD3DSettings newSettings ) を用いて生成すること。</para>
		/// <para>エラー等でアプリを終了したい場合は例外を発生させ、正常に（無言で）終了したい場合は this.Window または this.Direct3D9Device を null にして return すること。</para>
		/// </summary>
		protected override void On初期化()
		{
			#region [ プライマリアダプタのHALとフォーマットのチェックを行う。]
			//-----------------
			if( !this.Direct3D.CheckDeviceType( 0, SlimDX.Direct3D9.DeviceType.Hardware, Format.X8R8G8B8, Format.X8R8G8B8, true ) ||	// ウィンドウモード
				!this.Direct3D.CheckDeviceType( 0, SlimDX.Direct3D9.DeviceType.Hardware, Format.X8R8G8B8, Format.X8R8G8B8, false ) )	// 全画面モード
			{
				string msg = "プライマリディスプレイアダプタが、本ソフトの動作に必要な機能を満たしていません。";
				MessageBox.Show( msg, "StrokeStyle<T> エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
				throw new Exception( msg );
			}
			//-----------------
			#endregion

			#region [ Global.* を初期化する。]
			//-----------------
			//Global.t初期化( this );	--> コンストラクタへ移動
			//-----------------
			#endregion


			// 最初のユーザを選択する。

			string str最初のユーザ名 = null;

			#region [ 現状ユーザ数が 0 人、または SSTFEditor用ユーザが存在していないなら、SSTFEditor用ユーザを作成する。]
			//-----------------
			bool bSSTFEditor用ユーザを作成する = false;

			#region [ SSTFEditor用ユーザの作成は必要か？ ]
			//-----------------
			if( Global.Users.User.Count == 0 )
			{
				bSSTFEditor用ユーザを作成する = true;
			}
			else
			{
				bSSTFEditor用ユーザを作成する = true;
				foreach( var name in Global.Users.User )
				{
					if( name.Equals( Properties.Resources.DEFAULT_VIEWER_USER_NAME, StringComparison.OrdinalIgnoreCase ) )
					{
						bSSTFEditor用ユーザを作成する = false;		// ユーザがあった（ので作らない）
						break;
					}
				}
			}
			//-----------------
			#endregion

			if( bSSTFEditor用ユーザを作成する )
			{
				#region [ 必要だったので作成する。]
				//-----------------
				try
				{
					Global.Users.tユーザを新規追加しXMLファイルを保存する( Properties.Resources.DEFAULT_VIEWER_USER_NAME, Global.Folder.strUsersXMLの絶対パス );
				}
				catch( Exception e )
				{
					MessageBox.Show( "ユーザの登録に失敗しました。終了します。\n--------\n" + e.ToString(), "StrokeStyle<T>エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );
					throw;
				}
				//-----------------
				#endregion
			}
			//-----------------
			#endregion
			#region [ ユーザ選択ダイアログで最初のユーザを決定する。ログインはまだ。]
			//-----------------
			if( Global.PlayerMode.bプレイヤーモードである )
			{
				str最初のユーザ名 = Properties.Resources.DEFAULT_VIEWER_USER_NAME;
			}
			else
			{
				if( Global.Users.User.Count == 1 )	// SSTFEditor しかいない場合
				{
					// ユーザ登録

					#region [ 最初のユーザ名を起動者に入力させる。キャンセルされたらアプリ終了。]
					//-----------------
					str最初のユーザ名 = Global.Users.tユーザ登録ダイアログを表示し入力されたユーザ名を返す();

					if( string.IsNullOrEmpty( str最初のユーザ名 ) )
						return;		// ウィンドウ生成前に return すればアプリは終了する。
					//-----------------
					#endregion
				}
				else
				{
					// ユーザ選択

					#region [ 既存ユーザを選択させる。キャンセルされたらアプリ終了。]
					//-----------------
					Global.Stage.m演奏ステージ.t演奏レーンチップ対応表をConfigに合わせて再構築する();

					str最初のユーザ名 = Global.Users.tユーザ管理ダイアログを表示し選択されたユーザ名を返す();

					if( string.IsNullOrEmpty( str最初のユーザ名 ) )
						return;		// ウィンドウ生成前に return すればアプリは終了する。
					//-----------------
					#endregion
				}
			}
			//-----------------
			#endregion


			// 最初のユーザでログインする。

			this.tユーザのログイン処理( str最初のユーザ名 );	// ※以下、C共通.LogBlock() がログファイルに出力される。


			// その他、ユーザに依存しない一回限りの初期化を行う。

			#region [ メインウィンドウに対する入力を初期化する。]
			//-----------------
			Global.Input.Input管理 = new CInput管理( this.Window.Handle );
			//-----------------
			#endregion
			#region [ トップレベルActivitiesを生成・登録する。]
			//-----------------
			this.listトップレベルActivities = new List<CActivity>();
			this.listトップレベルActivities.Add( Global.Act英数字 );
			this.listトップレベルActivities.Add( Global.Actフリップボード );
			this.listトップレベルActivities.Add( Global.Stage.m演奏ステージ );
			this.listトップレベルActivities.Add( Global.Stage.m選曲ステージ );
			this.listトップレベルActivities.Add( Global.Stage.m結果ステージ );
			//-----------------
			#endregion
			#region [ ステージ以外のトップレベルActivityを活性化する。]
			//-----------------
			Global.Act英数字.On活性化();
			Global.Actフリップボード.On活性化();
			//-----------------
			#endregion


			// ステージ起動。

			#region [ 最初のステージを起動する。]
			//-----------------
			if( Global.PlayerMode.bプレイヤーモードではない )
			{
				// (A) 通常モードの場合
				Global.Stage.m現在のステージ = Global.Stage.m選曲ステージ;
				Global.Stage.m現在のステージ.On活性化();
			}
			else
			{
				// (B) プレイヤーモードの場合
				Global.Song.r現在演奏中の曲のスコア = null;		// プレイヤーモード時はスコア = null のまま演奏ステージを活性化する。
				Global.Stage.m現在のステージ = Global.Stage.m演奏ステージ;
				Global.Stage.m現在のステージ.On活性化();
			}
			//-----------------
			#endregion
		}

		/// <summary>
		/// <para>終了処理を行う。</para>
		/// <para>フレームワークにより、メインループを抜けた後、一番最後に呼び出される。</para>
		/// <para>Direct3D, Device は Dispose 不要。</para>
		/// </summary>
		protected override void On終了()
		{
			#region [ ログアウト ]
			//-----------------
			if( !string.IsNullOrEmpty( Global.Users.str現在のユーザ名 ) )
				this.tユーザのログアウト処理();
			//-----------------
			#endregion

			// ※リスナが削除されたので、以下はログファイルに出力されない。

			#region [ 全てのトップレベルActivityの非活性化（ステージ込み）]
			//-----------------
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.On非活性化();
			}
			//-----------------
			#endregion

			Global.t終了();

			C共通.t完全なガベージコレクションを実施する();
		}

		/// <summary>
		/// <para>進行処理を行う。</para>
		/// <para>BeginScene～EndScene の間に行えない処理もここで行う。</para>
		/// </summary>
		protected override void On進行()
		{
			if( this.bアプリケーションを終了する )	// 終了系WMイベント処理待ち
				return;

		// ステージが変わった場合の再処理開始ポイント。
		// → ステージが変わった場合は必ずここに戻ってくるようにする。
		// （このメソッドを抜けた直後の On描画() が行われるよりも 前 に 最 低 1 回 は進行を行う必要があるため。）

		L再進行開始ポイント:

			#region [ 入力のポーリング。]
			//-----------------
			if( Global.Input.Input管理.hWnd == Global.App.Window.Handle )	// 自分が Input管理 のオーナーであるときのみ更新。
				Global.Input.t入力のポーリングとキー入力イベントへの変換( Global.bウィンドウがアクティブである, Global.Users.Config.BufferedInput );
			//-----------------
			#endregion

			#region [ プレイヤーモードの場合、コマンドライン配列を定期的にチェックし、必要有ればアクションを実行する。]
			//-----------------
			if( Global.PlayerMode.bプレイヤーモードである )
			{
				if( Global.PlayerMode.arrayコマンドライン != null )		// 非 null なら実行。
				{
					Global.PlayerMode.tアクション実行( Global.PlayerMode.arrayコマンドライン );
					Global.PlayerMode.arrayコマンドライン = null;		// 実行後、null に戻す。
				}
			}
			//-----------------
			#endregion


			// ステージ以外のトップレベルActivityを進行。

			Global.Actフリップボード.On進行();


			// 現在のステージの進行。

			int n戻り値 = Global.Stage.m現在のステージ.On進行();


			// 戻り値で処理分岐。

			if( Global.PlayerMode.bプレイヤーモードではない )
			{
				#region [ (A) 通常モードの場合：結果によりステージを遷移する。]
				//-----------------
				switch( Global.Stage.m現在のステージ.eStageID )
				{
					case CStageBase.EステージID.選曲:

						#region [ 曲決定 ]
						//-----------------
						if( n戻り値 == (int) C選曲ステージ.E進行の戻り値.曲決定 )
						{
							// ここの時点で、
							// 「Global.Song.r現在演奏中の曲」の設定までは完了済みであり、
							// 「Global.Song.r現在演奏中の曲のスコア」は解放済みであること。


							// 選択された曲に対するスコアを構築。

							Global.Song.r現在演奏中の曲のスコア = Global.Song.tスコアを読み込む( Global.Song.r現在演奏中の曲.ScoreFile );


							// 構築の成否で分岐。

							if( Global.Song.r現在演奏中の曲のスコア != null )
							{
								#region [ (A) 読み込み成功 → 演奏ステージへ移行。]
								//-----------------

								// 選曲ステージ終了。

								Global.Stage.m現在のステージ.On非活性化();


								// 演奏ステージへ移行。

								Global.Stage.m現在のステージ = Global.Stage.m演奏ステージ;
								Global.Stage.m現在のステージ.On活性化();
								//-----------------
								#endregion
							}
							else
							{
								#region [ (B) 読み込み失敗 → フリップで失敗メッセージを出し、もう一度演奏ステージをやりなおす。]
								//-----------------

								// 選曲ステージ終了。

								Global.Stage.m現在のステージ.On非活性化();


								// 選曲ステージ再び活性化。

								Global.Stage.m現在のステージ = Global.Stage.m選曲ステージ;
								Global.Stage.m現在のステージ.On活性化();

		
								// エラーフリップ表示。

								Global.Actフリップボード.t表示( "曲の読み込みに失敗しました。" );
								//-----------------
								#endregion
							}

							goto L再進行開始ポイント;
						}
						//-----------------
						#endregion

						#region [ 終了 ]
						//-----------------
						else if( n戻り値 == (int) C選曲ステージ.E進行の戻り値.終了 )
						{
							// 選曲ステージ終了。

							Global.Stage.m現在のステージ.On非活性化();
							Global.Stage.m現在のステージ = null;


							// アプリ終了。

							this.bアプリケーションを終了する = true;
						}
						//-----------------
						#endregion

						break;

					case CStageBase.EステージID.演奏:

						#region [ クリア ]
						//-----------------
						if( n戻り値 == (int) C演奏ステージ.E戻り値.Clear )
						{
							// 演奏ステージ終了。

							//Global.Misc.Act背景動画.On非活性化();		// 演奏クリアの場合は、まだ再生停止しない。
							Global.Stage.m現在のステージ.On非活性化();


							// 結果ステージへ移行。

							Global.Stage.m現在のステージ = Global.Stage.m結果ステージ;
							Global.Stage.m現在のステージ.On活性化();

							goto L再進行開始ポイント;
						}
						//-----------------
						#endregion

						#region [ 演奏中止 ]
						//-----------------
						else if( n戻り値 == (int) C演奏ステージ.E戻り値.演奏中止 )
						{
							// 背景動画の停止と解放。
							// 演奏中止の場合は、すぐに再生停止・解放する。

							Global.Song.r現在演奏中のスコアの背景動画.t再生停止();
							Global.Song.r現在演奏中のスコアのBGM.tサウンドを停止する();
							C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアの背景動画 );
							C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアのBGM );


							// 演奏ステージ終了。

							Global.Stage.m現在のステージ.On非活性化();
							C共通.t完全なガベージコレクションを実施する();


							// 選曲ステージへ移行。

							Global.Stage.m現在のステージ = Global.Stage.m選曲ステージ;
							Global.Stage.m現在のステージ.On活性化();

							goto L再進行開始ポイント;
						}
						//-----------------
						#endregion

						break;

					case CStageBase.EステージID.結果:

						#region [ 終了 ]
						//-----------------
						if( n戻り値 == (int) C結果ステージ.E戻り値.終了 )
						{
							// ここで背景動画を停止。（→ 結果画面が終了するまで演奏をひっぱる。）

							Global.Song.r現在演奏中のスコアの背景動画.t再生停止();
							Global.Song.r現在演奏中のスコアのBGM.tサウンドを停止する();
							C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアの背景動画 );
							C共通.tDisposeする( ref Global.Song.r現在演奏中のスコアのBGM );


							// 結果ステージ終了。

							Global.Stage.m現在のステージ.On非活性化();
							C共通.t完全なガベージコレクションを実施する();


							// 選曲ステージへ移行。

							Global.Stage.m現在のステージ = Global.Stage.m選曲ステージ;
							Global.Stage.m現在のステージ.On活性化();

							goto L再進行開始ポイント;
						}
						//-----------------
						#endregion

						break;
				}
				//-----------------
				#endregion
			}
			else
			{
				#region [ (B) プレイヤーモードの場合： 延々と演奏ステージ。]
				//-----------------
				switch( Global.Stage.m現在のステージ.eStageID )
				{
					case CStageBase.EステージID.演奏:
						break;		// 永遠にステージ移行しない。
				}
				//-----------------
				#endregion
			}


			// 何らかの遅延指示が出ている場合は、それを行う。

			if( Global.b次のタイミングでD3Dデバイスを変更する )
			{
				#region [ Direct3D9デバイスを変更する。]
				//-----------------
				// D3Dデバイスの変更は実行タイミングがシビアなので、
				// 必ず遅延指示を出し、状況発生現場ではなくここで行うようにすること。


				// Direct3D9デバイスを変更する。

				var newSettings = this.currentD3DSettings.Clone();
				newSettings.PresentParameters.Windowed = Global.Users.Config.Windowed;
				newSettings.PresentParameters.PresentationInterval = ( Global.Users.Config.VSyncWait ) ? PresentInterval.Default : PresentInterval.Immediate;
				newSettings.PresentParameters.BackBufferWidth = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Width : Global.Theme.FullscreenSize.Width;
				newSettings.PresentParameters.BackBufferHeight = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize.Height : Global.Theme.FullscreenSize.Height;
				this.tDirect3Dデバイスを生成・変更・リセットする(		// 例外はキャッチしない。準正常じゃなくて異常なので。
					newSettings,
					Global.Theme.LogicalDisplaySize,
					CApplicationForm.wsウィンドウスタイル,
					CApplicationForm.ws全画面スタイル,
					true );


				// 完了。

				Global.b次のタイミングでD3Dデバイスを変更する = false;
				//-----------------
				#endregion
			}
		}

		/// <summary>
		/// <para>描画処理を行う。</para>
		/// <para>BeginScene() と EndScene() の間に呼び出される。</para>
		/// <para>そのため、Direct3Dデバイスの変更を伴うような操作は行わないこと。</para>
		/// </summary>
		protected override void On描画()
		{
			if( this.bアプリケーションを終了する )	// 終了系WMイベント処理待ち
				return;


			// 現在のステージの描画。

			if( Global.Stage.m現在のステージ != null )
				Global.Stage.m現在のステージ.On描画();


			// その他のトップレベルActivityの描画。

			Global.Actフリップボード.On描画();


			// 描画範囲外をマスキング。

			#region [ 論理画面以外の部分を黒テクスチャで覆い隠す。]
			//-----------------
			//       　物理画面
			// ┌─┬──────┬─┐
			// │　│　　　上　　│　│		左図の上下左右の部分を
			// │　├──────┤　│		４回に分けて塗りつぶす。
			// │　│　論理画面　│　│
			// │左│（物理画面　│右│		領域は論理画面の座標で示すことに注意。
			// │　│　描画領域）│　│
			// │　├──────┤　│
			// │　│　　　下　　│　│
			// └─┴──────┴─┘

			// 以下、マネージドメモリの消費を抑えるため、new Rectangle() は使わない。

			// 左
			if( CTexture.rc物理画面描画領域.Left > 0 )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 右
			if( CTexture.rc物理画面描画領域.Right < CTexture.sz物理画面.Width )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Right;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz物理画面.Width - CTexture.rc物理画面描画領域.Right;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 上
			if( CTexture.rc物理画面描画領域.Top > 0 )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = 0;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz論理画面.Width;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.rc物理画面描画領域.Top;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}

			// 下
			if( CTexture.rc物理画面描画領域.Bottom < CTexture.sz物理画面.Height )
			{
				this.rc黒タイルで塗りつぶす物理画面領域.X = CTexture.rc物理画面描画領域.Left;
				this.rc黒タイルで塗りつぶす物理画面領域.Y = CTexture.rc物理画面描画領域.Bottom;
				this.rc黒タイルで塗りつぶす物理画面領域.Width = CTexture.sz論理画面.Width;
				this.rc黒タイルで塗りつぶす物理画面領域.Height = CTexture.sz物理画面.Height - CTexture.rc物理画面描画領域.Bottom;
				this.t黒タイルテクスチャで領域を塗りつぶす( ref this.rc黒タイルで塗りつぶす物理画面領域 );
			}
			//-----------------
			#endregion
		}

		/// <summary>
		/// <para>Managed リソースを生成する。</para>
		/// <para>フレームワークにより、デバイスを生成した直後に呼び出される。（リセットの直後には呼び出されない。）</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnManageリソースを生成する()
		{
			#region [ 黒タイルテクスチャの生成 ]
			//-----------------
			using( var bmp = new Bitmap( 256, 256 ) )
			using( var g = Graphics.FromImage( bmp ) )
			{
				g.FillRectangle( Brushes.Black, 0, 0, bmp.Width, bmp.Height );
				this.tx黒タイル = Global.tテクスチャを生成する( bmp );
			}
			//-----------------
			#endregion

			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnManagedリソースの作成();
			}
		}

		/// <summary>
		/// <para>Managed リソースを解放する。</para>
		/// <para>デバイスを破棄する直前に呼び出される。（リセットの直前には呼び出されない。）</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnManageリソースを解放する()
		{
			C共通.tDisposeする( ref this.tx黒タイル );

			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnManagedリソースの解放();
			}
		}

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを生成する。</para>
		/// <para>フレームワークにより、デバイスロストした直後に呼び出される。</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnUnmanageリソースを生成する()
		{
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnUnmanagedリソースの作成();
			}
		}

		/// <summary>
		/// <para>Unmanaged リソースがあれば、それを解放する。</para>
		/// <para>フレームワークにより、デバイスをリセットまたは破棄する直前に呼び出される。</para>
		/// <para>連続して呼び出されても問題ないようにコーディングすること。</para>
		/// </summary>
		protected override void OnUnmanageリソースを解放する()
		{
			if( this.listトップレベルActivities != null )
			{
				foreach( CActivity act in this.listトップレベルActivities )
					act.OnUnmanagedリソースの解放();
			}
		}

		/// <summary>
		/// <para>Direct3Dデバイス（this.Device）に対するデフォルト設定を行う。</para>
		/// <para>Direct3Dデバイスのリセット・変更・再作成時に呼び出される。</para>
		/// </summary>
		protected override void OnD3Dデバイスステータスの初期化()
		{
			Size sz論理画面 = Global.Theme.LogicalDisplaySize;
			Size sz物理画面 = ( Global.Users.Config.Windowed ) ? Global.Theme.WindowClientSize : Global.Theme.FullscreenSize;

			D3D9Device.SetTransform( TransformState.View,
				Matrix.LookAtLH(
					new Vector3( 0f, 0f, (float) ( ( -sz物理画面.Height / 2.0 ) * Math.Sqrt( 3.0 ) ) ),			// カメラの位置
					new Vector3( 0.0f, 0.0f, 0.0f ),															// 見ているターゲット
					new Vector3( 0.0f, 1.0f, 0.0f ) ) );														// 上方向
			
			D3D9Device.SetTransform( TransformState.Projection,
				Matrix.PerspectiveFovLH(
					C共通.DegreeToRadian( 60.0f ),																// y 方向の視野角度（ラジアン）
					(float) sz物理画面.Width / sz物理画面.Height,												// アスペクト比。View の幅÷高さ。
					-100.0f,																					// z（近）
					100.0f ) );																					// z（遠）
			
			D3D9Device.SetRenderState( RenderState.Lighting, false );
			D3D9Device.SetRenderState( RenderState.ZEnable, false );
			D3D9Device.SetRenderState( RenderState.AntialiasedLineEnable, false );
			D3D9Device.SetRenderState( RenderState.AlphaTestEnable, true );
			D3D9Device.SetRenderState( RenderState.AlphaRef, 10 );
			D3D9Device.SetRenderState( RenderState.MultisampleAntialias, false );
			D3D9Device.SetRenderState<Compare>( RenderState.AlphaFunc, Compare.Greater );
			D3D9Device.SetRenderState( RenderState.AlphaBlendEnable, true );
			D3D9Device.SetRenderState<Blend>( RenderState.SourceBlend, Blend.SourceAlpha );
			D3D9Device.SetRenderState<Blend>( RenderState.DestinationBlend, Blend.InverseSourceAlpha );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaOperation, TextureOperation.Modulate );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaArg1, 2 );
			D3D9Device.SetTextureStageState( 0, TextureStage.AlphaArg2, 1 );
		}
		//-----------------
		#endregion

		#region [ protected ]
		//-----------------
		protected List<CActivity> listトップレベルActivities = null;
		protected CTexture tx黒タイル = null;
		protected Rectangle rc黒タイルで塗りつぶす物理画面領域 = new Rectangle();
		
		protected void t黒タイルテクスチャで領域を塗りつぶす( ref Rectangle rc塗りつぶす物理画面領域 )
		{
			// ※ 引数が ref なのは高速化のため。

			if( this.tx黒タイル == null )
				return;

			int nX方向枚数 = rc塗りつぶす物理画面領域.Width / this.tx黒タイル.sz画像サイズ.Width;
			int nX方向余り = rc塗りつぶす物理画面領域.Width % this.tx黒タイル.sz画像サイズ.Width;
			if( nX方向余り != 0 )
				nX方向枚数++;

			int nY方向枚数 = rc塗りつぶす物理画面領域.Height / this.tx黒タイル.sz画像サイズ.Height;
			int nY方向余り = rc塗りつぶす物理画面領域.Height % this.tx黒タイル.sz画像サイズ.Height;
			if( nY方向余り != 0 )
				nY方向枚数++;

			for( int y = 0; y < nY方向枚数; y++ )
			{
				var rc黒タイル = new Rectangle( 0, 0, this.tx黒タイル.sz画像サイズ.Width, this.tx黒タイル.sz画像サイズ.Height );

				if( ( nY方向余り != 0 ) && ( y == nY方向枚数 - 1 ) )
					rc黒タイル.Height = nY方向余り;		// 最後の1枚が余りの場合は縦サイズが違う

				for( int x = 0; x < nX方向枚数; x++ )
				{
					if( ( nX方向余り != 0 ) && ( x == nX方向枚数 - 1 ) )
						rc黒タイル.Width = nX方向余り;	// 最後の1枚が余りの場合は横サイズが違う

					tx黒タイル.t物理2D描画(		// 描画。
						this.D3D9Device,
						rc塗りつぶす物理画面領域.X + x * this.tx黒タイル.sz画像サイズ.Width,
						rc塗りつぶす物理画面領域.Y + y * this.tx黒タイル.sz画像サイズ.Height,
						rc黒タイル );
				}
			}
		}
		//-----------------
		#endregion

		#region [ イベント ]
		//-----------------
		protected void OnResizeEnd( object sender, EventArgs e )
		{
			if( Global.Users.Config.Windowed == false )
				return;		// （たぶん来ないだろうが）全画面モードでは無視。

			Global.Users.Config.WindowClientSize =		// Config と
				Global.Theme.WindowClientSize =			// Theme の両方へ反映。
				this.Window.ClientSize;

			Global.b次のタイミングでD3Dデバイスを変更する = true;
		}
		//-----------------
		#endregion

	}
}
