﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Drawing;
using System.Xml.Serialization;
using System.Windows.Forms;
using FDK;

namespace StrokeStyleT
{
	public class KeyAssign : ICloneable
	{
		// クラス定義

		public class Key : ICloneable
		{
			public string DeviceGUID;
			public int DeviceID;
			public int Note;

			#region [ ICloneable 実装 ]
			//-----------------
			public object Clone()
			{
				return (object) new Key() {
					DeviceGUID = this.DeviceGUID,
					DeviceID = this.DeviceID,
					Note = this.Note,
				};
			}
			//-----------------
			#endregion
		}
		public class Pad : ICloneable
		{
			public E入力アサインパッド PadType;
			public List<Key> Keys;

			#region [ ICloneable 実装 ]
			//-----------------
			public object Clone()
			{
				var clone = new Pad();

				clone.PadType = this.PadType;
				clone.Keys = new List<Key>();
				foreach( var key in this.Keys )
					clone.Keys.Add( (Key) key.Clone() );

				return (object) clone;
			}
			//-----------------
			#endregion
		}


		// プロパティ（保存される）

		public List<Pad> Pads = new List<Pad>();
		public int FootPedalMin = -1;
		public int FootPedalMax = -1;

	
		// プロパティ（保存されない）

		[XmlIgnore]
		internal Dictionary<Eチップ, Pad> dicKeyAssigns = new Dictionary<Eチップ, Pad>();


		// メソッド（保存されない）

		public KeyAssign()
		{
		}
		internal static KeyAssign t読み込む( string fileName )
		{
			// 読み込み。
			
			KeyAssign assign = null;
			try
			{
				C共通.tXMLファイルを読み込む( fileName, out assign );
			}
			catch( FileNotFoundException )
			{
				Trace.TraceWarning( "ファイルが存在しません。規定値を使用します。[{0}]", Global.Folder.tファイルパスをマクロ付きパスに逆展開する( fileName ) );
				assign = new KeyAssign();
			}
			catch( Exception e )
			{
				C共通.t例外の詳細をログに出力する( e );
				Trace.TraceWarning( "ファイルの読み込みに失敗しました。規定値を使用します。[{0}]", Global.Folder.tファイルパスをマクロ付きパスに逆展開する( fileName ) );
				assign = new KeyAssign();
			}


			// 後処理。

			KeyAssign.t現在のPadsをdicKeyAssignに割り当てる( assign );

			return assign;
		}
		internal void t初期値を設定する()
		{
			this.FootPedalMin = -1;
			this.FootPedalMax = -1;
			
			this.Pads.Clear();

			// ここで行っているのは最小限の設定。以下同じ。

			#region [ LeftCymbal ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.LeftCymbal,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 49 },
					new Key() { DeviceID = 0, Note = 57 },
					new Key() { DeviceID = 1, Note = 49 },
					new Key() { DeviceID = 1, Note = 57 },
				}
			} );
			//-----------------
			#endregion
			#region [ FootPedal ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.FootPedal,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 44 },
					new Key() { DeviceID = 1, Note = 44 },
				}
			} );
			//-----------------
			#endregion
			#region [ HiHat ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.HiHat,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 42 },
					new Key() { DeviceID = 0, Note = 46 },
					new Key() { DeviceID = 1, Note = 44 },
					new Key() { DeviceID = 1, Note = 46 },
				}
			} );
			//-----------------
			#endregion
			#region [ Snare ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Snare,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 38 },
					new Key() { DeviceID = 0, Note = 40 },
					new Key() { DeviceID = 1, Note = 38 },
					new Key() { DeviceID = 1, Note = 40 },
				}
			} );
			//-----------------
			#endregion
			#region [ SnareOpenRim ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.SnareOpenRim,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ SnareClosedRim ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.SnareClosedRim,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ Tom1 ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom1,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 48 },
					new Key() { DeviceID = 0, Note = 50 },
					new Key() { DeviceID = 1, Note = 48 },
					new Key() { DeviceID = 1, Note = 50 },
				}
			} );
			//-----------------
			#endregion
			#region [ Tom1Rim ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom1Rim,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ Bass ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Bass,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 33 },
					new Key() { DeviceID = 0, Note = 35 },
					new Key() { DeviceID = 0, Note = 36 },
					new Key() { DeviceID = 1, Note = 33 },
					new Key() { DeviceID = 1, Note = 35 },
					new Key() { DeviceID = 1, Note = 36 },
				}
			} );
			//-----------------
			#endregion
			#region [ Tom2 ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom2,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 45 },
					new Key() { DeviceID = 0, Note = 47 },
					new Key() { DeviceID = 1, Note = 45 },
					new Key() { DeviceID = 1, Note = 47 },
				}
			} );
			//-----------------
			#endregion
			#region [ Tom2Rim ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom2Rim,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ Tom3 ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom3,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 41 },
					new Key() { DeviceID = 0, Note = 43 },
					new Key() { DeviceID = 1, Note = 41 },
					new Key() { DeviceID = 1, Note = 43 },
				}
			} );
			//-----------------
			#endregion
			#region [ Tom3Rim ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Tom3Rim,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ RightCymbal ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.RightCymbal,
				Keys = new List<Key>(),
			} );
			//-----------------
			#endregion
			#region [ RideCymbal ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.RideCymbal,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 51 },
					new Key() { DeviceID = 0, Note = 59 },
					new Key() { DeviceID = 1, Note = 51 },
					new Key() { DeviceID = 1, Note = 59 },
				}
			} );
			//-----------------
			#endregion
			#region [ RideCup ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.RideCup,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 53 },
					new Key() { DeviceID = 1, Note = 53 },
				}
			} );
			//-----------------
			#endregion
			#region [ ChinaCymbal ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.ChinaCymbal,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 52 },
					new Key() { DeviceID = 1, Note = 52 },
				}
			} );
			//-----------------
			#endregion
			#region [ Splash ]
			//-----------------
			this.Pads.Add( new Pad() {
				PadType = E入力アサインパッド.Splash,
				Keys = new List<Key>() {
					new Key() { DeviceID = 0, Note = 55 },
					new Key() { DeviceID = 1, Note = 55 },
				}
			} );
			//-----------------
			#endregion

			KeyAssign.t現在のPadsをdicKeyAssignに割り当てる( this );
		}
		internal void t保存する( string fileName )
		{
			try
			{
				C共通.tXMLファイルを保存する( fileName, this );
			}
			catch( Exception e )
			{
				MessageBox.Show( "ファイルの保存に失敗しました。[{0}]\n--------\n" + e.ToString(),
					Global.Folder.tファイルパスをマクロ付きパスに逆展開する( fileName ) );
			}
		}

		#region [ ICloneable 実装 ]
		//-----------------
		public object Clone()
		{
			var ks = new KeyAssign();

			ks.FootPedalMin = this.FootPedalMin;
			ks.FootPedalMax = this.FootPedalMax;

			#region [ KeyAssign.Pads のコピー ]
			//-----------------
			ks.Pads = new List<Pad>();
			foreach( var item in this.Pads )
				ks.Pads.Add( (Pad) item.Clone() );
			//-----------------
			#endregion
			#region [ KeyAssign.dicKeyAssign のコピー ]
			//-----------------
			KeyAssign.t現在のPadsをdicKeyAssignに割り当てる( ks );
			//-----------------
			#endregion

			return (object) ks;
		}
		//-----------------
		#endregion

		#region [ protected ]
		//-----------------
		protected static void t現在のPadsをdicKeyAssignに割り当てる( KeyAssign assign )
		{
			assign.dicKeyAssigns = new Dictionary<Eチップ, Pad>() {
				{ Eチップ.LeftCrash, assign.Pads[(int) E入力アサインパッド.LeftCymbal ] },
				{ Eチップ.HiHat_Foot, assign.Pads[(int) E入力アサインパッド.FootPedal ] },
				{ Eチップ.HiHat_Close, assign.Pads[(int) E入力アサインパッド.HiHat ] },		// Close, HalfOpen, Open はすべて同一キーバインド
				{ Eチップ.HiHat_HalfOpen, assign.Pads[(int) E入力アサインパッド.HiHat ] },	//
				{ Eチップ.HiHat_Open, assign.Pads[(int) E入力アサインパッド.HiHat ] },		//
				{ Eチップ.Snare, assign.Pads[(int) E入力アサインパッド.Snare ] },
				{ Eチップ.Snare_Ghost, assign.Pads[(int) E入力アサインパッド.Snare ] },
				{ Eチップ.Snare_OpenRim, assign.Pads[(int) E入力アサインパッド.SnareOpenRim ] },
				{ Eチップ.Snare_ClosedRim, assign.Pads[(int) E入力アサインパッド.SnareClosedRim ] },
				{ Eチップ.Tom1, assign.Pads[(int) E入力アサインパッド.Tom1 ] },
				{ Eチップ.Tom1_Rim, assign.Pads[(int) E入力アサインパッド.Tom1Rim ] },
				{ Eチップ.Bass, assign.Pads[(int) E入力アサインパッド.Bass ] },
				{ Eチップ.Tom2, assign.Pads[(int) E入力アサインパッド.Tom2 ] },
				{ Eチップ.Tom2_Rim, assign.Pads[(int) E入力アサインパッド.Tom2Rim ] },
				{ Eチップ.Tom3, assign.Pads[(int) E入力アサインパッド.Tom3 ] },
				{ Eチップ.Tom3_Rim, assign.Pads[(int) E入力アサインパッド.Tom3Rim ] },
				{ Eチップ.RightCrash, assign.Pads[(int) E入力アサインパッド.RightCymbal ] },
				{ Eチップ.Ride, assign.Pads[(int) E入力アサインパッド.RideCymbal ] },
				{ Eチップ.Ride_Cup, assign.Pads[(int) E入力アサインパッド.RideCup ] },
				{ Eチップ.China, assign.Pads[(int) E入力アサインパッド.ChinaCymbal ] },
				{ Eチップ.Splash, assign.Pads[(int) E入力アサインパッド.Splash ] },
			};
		}
		//-----------------
		#endregion
	}
}
