﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;
using FDK;

namespace StrokeStyleT
{
	static class Program
	{
		[STAThread]
		static void Main()
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault( false );

			#region [ 二重起動しているかチェックし、既に起動しているなら起動済みのexeにコマンドラインを送信する。]
			//-----------------
			var mutex = new Mutex( false, Properties.Resources.MUTEX_NAME );

			if( !mutex.WaitOne( 0, false ) )
			{
				// 実行中の StrokeStyle がプレイヤーモードであれば、そのウィンドウにコマンドラインを送信して終了する。

				var hWnd = CWin32.FindWindow( null, Properties.Resources.TITLE_PLAYERMODE );	// Form はクラス名が一定でなく変更もできないので、ウィンドウタイトルで判断する。
				if( hWnd != IntPtr.Zero )
				{
					string[] cmdArgs = Environment.GetCommandLineArgs();
					if( cmdArgs.Length > 1 )	// cmdArgs[0] は .exe のパス
					{
						#region [ cmdArgs[0] を除いたコマンドライン msg を再構築。]
						//-----------------
						var sb = new StringBuilder();

						for( int i = 1; i < cmdArgs.Length; i++ )
							sb.Append( cmdArgs[ i ] + Environment.NewLine );		// 改行で区切る。

						string msg = sb.ToString();
						//-----------------
						#endregion
						#region [ コマンドライン msg を既存 StrokeStyle へ送信。]
						//-----------------
						byte[] msgRow = Encoding.Default.GetBytes( msg );
						var copyData = new CWin32.COPYDATA() {
							dwData = 0,
							lpData = msg,
							cbData = msgRow.Length + 1,
						};
						CWin32.SendMessage( hWnd, CWin32.WM_COPYDATA, 0, ref copyData );
						//-----------------
						#endregion
					}
				}
				else
				{
					// プレイヤーモードじゃないなら何もせず終了。
				}

				return;		// 注：WaitOne() が false の場合、ReleaseMutex() は不要。
			}
			//-----------------
			#endregion

			#region [ アプリケーションの起動。]
			//-----------------
			try
			{
				using( var app = new App() )
				{
					#region [ コマンドライン引数があればプレイヤーモードとして動作する。]
					//-----------------
					string[] cmdArgs = Environment.GetCommandLineArgs();

					if( cmdArgs.Length > 1 )	// [0] は .exe のパスなので除外
					{
						Global.PlayerMode.bプレイヤーモードである = true;
						Global.PlayerMode.arrayコマンドライン = new string[ cmdArgs.Length - 1 ];
						for( int i = 1; i < cmdArgs.Length; i++ )
							Global.PlayerMode.arrayコマンドライン[ i - 1 ] = cmdArgs[ i ];

						// アクションの実行は、アプリの初期化が終わってから、Global.PlayerMode.arrayコマンドライン[] で判別して行う。
					}
					else
					{
						Global.PlayerMode.bプレイヤーモードである = false;
						Global.PlayerMode.arrayコマンドライン = null;
					}
					//-----------------
					#endregion

					app.Run();
				}

				// (A) 正常終了。

				Trace.WriteLine( "" );
				Trace.WriteLine( "遊んでくれてありがとう！" );
			}
#if !DEBUG	// デバッグ時はキャッチしない方が解析に便利。
			catch( Exception e )
			{
				// (B) 異常終了。キャッチされなかった例外をダイアログとログに出力。

				MessageBox.Show( string.Format( "未処理の例外が発生しました。\n終了します。\n\n{0}", e.ToString() ), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error );

				C共通.t例外の詳細をログに出力する( e );
				Trace.WriteLine( "" );
				Trace.WriteLine( "エラーだゴメン！（涙" );
			}
#endif
			finally
			{
			}
			//-----------------
			#endregion
			#region [ アプリケーションの終了。]
			//-----------------

			// .txt ログ出力リスナーが存在していれば削除する。

			for( int i = Trace.Listeners.Count - 1; i >= 1; i-- )
				Trace.Listeners.RemoveAt( i );


			// Mutex を解放。

			mutex.ReleaseMutex();	// mutex.WaitOne() が true を返した場合、ReleaseMutex() が必要。
			mutex = null;
			//-----------------
			#endregion

			// おしまい。
		}
	}
}
