﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Drawing;
using System.Xml.Serialization;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using FDK;
using Un4seen.BassAsio;
using Un4seen.BassWasapi;

namespace StrokeStyleT
{
	public class Config
	{
		// プロパティ（保存される）

		public string SSTVersion = Global.nリビジョン番号.ToString();
		public string CurrentKitName = "Default";		// 今のところ複数キット対応は未実装。ここを変更しても意味はない。
		public float Speed = 2.0f;
		public bool VSyncWait = false;
		public bool Windowed = true;
		public Size WindowClientSize = new Size( 0, 0 );
		public Size FullscreenSize = new Size( 0, 0 );
		public int FullscreenRefreshRate = 0;
		public bool AutoPlayLeftCymbal = false;
		public bool AutoPlayFootPedal = false;
		public bool AutoPlayHiHat = false;
		public bool AutoPlaySnare = false;
		public bool AutoPlayTom1 = false;
		public bool AutoPlayBass = false;
		public bool AutoPlayTom2 = false;
		public bool AutoPlayTom3 = false;
		public bool AutoPlayRightCymbal = false;
		public bool WriteLog = true;
		public string ThemeName = @"Default\";
		public bool LeftChina = false;
		public bool LeftRide = false;
		public bool LeftSplash = true;
		public bool BufferedInput = true;
		public bool HitSound = false;
		public List<string> SongFileExtensions = new List<string>();
		public int PerfectRange = int.Parse( Properties.Resources.DEFAULT_PERFECT_RANGE );
		public int GreatRange = int.Parse( Properties.Resources.DEFAULT_GREAT_RANGE );
		public int GoodRange = int.Parse( Properties.Resources.DEFAULT_GOOD_RANGE );
		public int PoorRange = int.Parse( Properties.Resources.DEFAULT_POOR_RANGE );
		/// <summary>
		/// <para>SongList.xml のパス。読み込み直後にマクロが展開され、保存直前にマクロに変換される。</para>
		/// </summary>
		public List<string> SongListFiles = new List<string>();
		/// <summary>
		/// <para>KitList.xml のパス。読み込み直後にマクロが展開され、保存直前にマクロに変換される。</para>
		/// </summary>
		public List<string> KitListFiles = new List<string>();
		public Eコンボ表示位置 ComboPosition = Eコンボ表示位置.LEFT;
		/// <summary>
		/// <para>演奏時のフレーム背景の不透明度。（完全透明:0.0～1.0:完全不透明）</para>
		/// </summary>
		public int FrameBackOpacity = 20;
		public ESoundDeviceType SoundDeviceType = ESoundDeviceType.Unknown;
		/// <summary>
		/// <para>WASAPI 排他モード出力における再生遅延[ms]（の希望値）。最終的にはこの数値を基にドライバが決定する）。</para>
		/// </summary>
		public int SoundDelayExclusiveWASAPI = 50;
		/// <summary>
		/// <para>WASAPI 共有モード出力における再生遅延[ms]。ユーザが決定する。</para>
		/// </summary>
		public int SoundDelaySharedWASAPI = 100;
		/// <summary>
		/// <para>排他WASAPIバッファの更新間隔。出力間隔ではないので注意。</para>
		/// <para>SoundDelay よりも小さい値であること。（小さすぎる場合はBASSによって自動修正される。）</para>
		/// </summary>
		public int SoundUpdatePeriodExclusiveWASAPI = 6;
		/// <summary>
		/// <para>共有WASAPIバッファの更新間隔。出力間隔ではないので注意。</para>
		/// <para>SoundDelay よりも小さい値であること。（小さすぎる場合はBASSによって自動修正される。）</para>
		/// </summary>
		public int SoundUpdatePeriodSharedWASAPI = 6;
		/// <summary>
		/// <para>ASIO 出力における再生遅延[ms]（の希望値）。最終的にはこの数値を基にドライバが決定する）。</para>
		/// </summary>
		public int SoundDelayASIO = 50;
		/// <summary>
		/// <para>DirectSound 出力における再生遅延[ms]。ユーザが決定する。</para>
		/// </summary>
		public int SoundDelayDirectSound = 100;


		// プロパティ（保存されない）

		[XmlIgnore]
		internal bool b全オートである
		{
			get
			{
				foreach( var kvp in this.dicAutoPlay )
				{
					if( kvp.Key == E演奏レーン.Unknown )
						continue;

					if( kvp.Value == false )
						return false;
				}
				return true;
			}
		}

		[XmlIgnore]
		internal Dictionary<E演奏レーン,bool> dicAutoPlay = new Dictionary<E演奏レーン, bool>() {
			{ E演奏レーン.LeftCymbal, false },
			{ E演奏レーン.FootPedal, false },
			{ E演奏レーン.HiHat, false },
			{ E演奏レーン.Snare, false },
			{ E演奏レーン.Tom1, false },
			{ E演奏レーン.Bass, false },
			{ E演奏レーン.Tom2, false },
			{ E演奏レーン.Tom3, false },
			{ E演奏レーン.RightCymbal, false },
			{ E演奏レーン.Unknown, false },
		};

		[XmlIgnore]
		internal Dictionary<E判定,int> dicRange = new Dictionary<E判定, int>() {
			{ E判定.PERFECT, int.Parse( Properties.Resources.DEFAULT_PERFECT_RANGE ) },
			{ E判定.GREAT, int.Parse( Properties.Resources.DEFAULT_GREAT_RANGE ) },
			{ E判定.GOOD, int.Parse( Properties.Resources.DEFAULT_GOOD_RANGE ) },
			{ E判定.POOR, int.Parse( Properties.Resources.DEFAULT_POOR_RANGE ) },
			{ E判定.MISS, 0 },
			{ E判定.AUTO, 0 },
		};

		public static List<string> listデフォルト拡張子 =	// SSTFEditor でも使うので public。
			new List<string>() { 
				".mp3", ".wav",	".ogg",								// 音楽ファイルが前
				".avi", ".flv", ".mp4", ".wmv", ".mpg", ".mpeg"		// 動画ファイルが後
			};


		// メソッド（保存されない）

		internal static Config t読み込む( string fileName )
		{
			Config config = null;
			bool bFileFound = true;

			#region [ Configファイルの読み込み ]
			//-----------------
			try
			{
				C共通.tXMLファイルを読み込む( fileName, out config );
			}
			catch( FileNotFoundException )
			{
				bFileFound = false;
				config = new Config();
			}
			catch( Exception e )
			{
				C共通.t例外の詳細をログに出力する( e );
				bFileFound = false;
				config = new Config();
			}
			//-----------------
			#endregion

			#region [ Config内容の妥当性確認、修正 ]
			//-----------------

			#region [ SongExtentions がないならデフォルト拡張子を付与する。]
			//-----------------
			if( config.SongFileExtensions == null ||
				config.SongFileExtensions.Count == 0 )
			{
				config.SongFileExtensions = Config.listデフォルト拡張子;
			}
			//-----------------
			#endregion

			if( bFileFound )
			{
				#region [ SongExtensions の先頭が '.' でないなら付与する。また、無効な文字列は排除する。]
				//-----------------
				for( int i = 0; i < config.SongFileExtensions.Count; i++ )
				{
					string ext = config.SongFileExtensions[ i ].Trim();

					if( ext[ 0 ] != '.' )
						config.SongFileExtensions[ i ] = '.' + ext;
				}

				// 空文字列はすべて除去する。

				config.SongFileExtensions.RemoveAll( ( path ) => { return string.IsNullOrEmpty( path ); } );
				//-----------------
				#endregion

				#region [ AutoPlay* を dicAutoPlay に反映する。]
				//-----------------
				config.dicAutoPlay = new Dictionary<E演奏レーン, bool>() {
					{ E演奏レーン.LeftCymbal, config.AutoPlayLeftCymbal },
					{ E演奏レーン.FootPedal, config.AutoPlayFootPedal },
					{ E演奏レーン.HiHat, config.AutoPlayHiHat },
					{ E演奏レーン.Snare, config.AutoPlaySnare },
					{ E演奏レーン.Tom1, config.AutoPlayTom1 },
					{ E演奏レーン.Bass, config.AutoPlayBass },
					{ E演奏レーン.Tom2, config.AutoPlayTom2 },
					{ E演奏レーン.Tom3, config.AutoPlayTom3 },
					{ E演奏レーン.RightCymbal, config.AutoPlayRightCymbal },
					{ E演奏レーン.Unknown, false },
				};
				//-----------------
				#endregion
				#region [ Range* を dicRange に反映する。]
				//-----------------
				config.dicRange = new Dictionary<E判定, int>() {
						{ E判定.PERFECT, config.PerfectRange },
						{ E判定.GREAT, config.GreatRange },
						{ E判定.GOOD, config.GoodRange },
						{ E判定.POOR, config.PoorRange },
						{ E判定.MISS, 0 },
						{ E判定.AUTO, 0 },
					};
				//-----------------
				#endregion

				#region [ SongListFiles 要素のマクロを展開し、パスが間違っている要素や、存在しない要素を削除する。]
				//-----------------
				for( int i = config.SongListFiles.Count - 1; i >= 0; i-- )
				{
					// 空文字列なら除去する。

					if( string.IsNullOrEmpty( config.SongListFiles[ i ] ) )
					{
						config.SongListFiles.RemoveAt( i );
						continue;
					}


					// パスが間違っているものや、存在していない場合には除去する。

					try
					{
						config.SongListFiles[ i ] = Global.Folder.tファイルパスのマクロを展開する( config.SongListFiles[ i ] );
						config.SongListFiles[ i ] = Global.Folder.tパスを絶対パスに変換しパスとしての正当性を確認する( config.SongListFiles[ i ].Trim(), Global.Folder.strシステムフォルダ );
					}
					catch
					{
						// 例外が発生したら無効な文字列ということなので無視する。
						config.SongListFiles.RemoveAt( i );
						continue;
					}
					if( !File.Exists( config.SongListFiles[ i ] ) )	// 引数がファイルじゃなくフォルダである場合にも false が返る。
					{
						// SongListファイルが存在しないので無視する。
						config.SongListFiles.RemoveAt( i );
						continue;
					}

				}
				//-----------------
				#endregion
				#region [ KitListFiles 要素のマクロを展開し、パスが間違っている要素や、存在しない要素を削除する。]
				//-----------------
				for( int i = config.KitListFiles.Count - 1; i >= 0; i-- )
				{
					// 空文字列なら除去する。

					if( string.IsNullOrEmpty( config.KitListFiles[ i ] ) )
					{
						config.KitListFiles.RemoveAt( i );
						continue;
					}


					// パスが間違っているものや、存在していない場合には除去する。

					try
					{
						config.KitListFiles[ i ] = Global.Folder.tファイルパスのマクロを展開する( config.KitListFiles[ i ] );
						config.KitListFiles[ i ] = Global.Folder.tパスを絶対パスに変換しパスとしての正当性を確認する( config.KitListFiles[ i ].Trim(), Global.Folder.strシステムフォルダ );
					}
					catch
					{
						// 例外が発生したら無効な文字列ということなので無視する。
						config.KitListFiles.RemoveAt( i );
						continue;
					}
					if( !File.Exists( config.KitListFiles[ i ] ) )			// 引数がファイルじゃなくフォルダである場合にも false が返る。
					{
						// KitListファイルが存在しないので無視する。
						config.KitListFiles.RemoveAt( i );
						continue;
					}
				}
				//-----------------
				#endregion

				#region [ Speed が 0.0 以下なら 1.0 にする。]
				//-----------------
				if( config.Speed <= 0.0f )
					config.Speed = 1.0f;
				//-----------------
				#endregion
			}

			#region [ SongListFilesが1つもない場合、システムフォルダにSongList.xmlがあればそれを加える。]
			//-----------------
			if( config.SongListFiles.Count == 0 )
			{
				string path = Path.Combine( Global.Folder.strシステムフォルダ, Properties.Resources.XMLNAME_SONG_LIST );

				if( File.Exists( path ) )
					config.SongListFiles.Add( path );
			}
			//-----------------
			#endregion
			#region [ KitListFilesが1つもない場合、システムフォルダにKitList.xmlがあればそれを加える。]
			//-----------------
			if( config.KitListFiles.Count == 0 )
			{
				string path = Path.Combine( Path.Combine( Global.Folder.strシステムフォルダ, @"DrumsKits\Default" ), Properties.Resources.XMLNAME_KITLIST );

				if( File.Exists( path ) )
					config.KitListFiles.Add( path );
			}
			//-----------------
			#endregion

			#region [ SoundDeviceType の指定がない場合、自動判別する。]
			//-----------------
			if( config.SoundDeviceType == ESoundDeviceType.Unknown )
			{
				if( BassWasapi.BASS_WASAPI_GetDeviceInfo( 0 ) != null )
				{
					config.SoundDeviceType = ESoundDeviceType.ExclusiveWASAPI;
				}
				else if( BassAsio.BASS_ASIO_GetDevice() != -1 )
				{
					config.SoundDeviceType = ESoundDeviceType.ASIO;
				}
				else
				{
					config.SoundDeviceType = ESoundDeviceType.DirectSound;
				}
			}
			//-----------------
			#endregion

			//-----------------
			#endregion

			return config;
		}
		internal void t保存する( string fileName )
		{
			#region [ SongListFiles 要素のシステム/ユーザフォルダパスを、マクロに置換する。]
			//-----------------
			for( int i = 0; i < this.SongListFiles.Count; i++ )
			{
				if( !string.IsNullOrEmpty( this.SongListFiles[ i ] ) )
					this.SongListFiles[ i ] = Global.Folder.tファイルパスをマクロ付きパスに逆展開する( this.SongListFiles[ i ] );
			}
			//-----------------
			#endregion
			#region [ KitListFiles 要素のシステム/ユーザフォルダパスを、マクロに置換する。]
			//-----------------
			for( int i = 0; i < this.KitListFiles.Count; i++ )
			{
				if( !string.IsNullOrEmpty( this.KitListFiles[ i ] ) )
					this.KitListFiles[ i ] = Global.Folder.tファイルパスをマクロ付きパスに逆展開する( this.KitListFiles[ i ] );
			}
			//-----------------
			#endregion

			try
			{
				C共通.tXMLファイルを保存する( fileName, this );
			}
			catch( Exception e )
			{
				MessageBox.Show( "ファイルの保存に失敗しました。[{0}]\n--------\n" + e.ToString(),
					Global.Folder.tファイルパスをマクロ付きパスに逆展開する( fileName ) );
			}

			// ここから継続して使う場合があるので、マクロを元の状態に戻しておく。

			#region [ SongListFiles 要素のシステム/ユーザフォルダパスのマクロを展開する。]
			//-----------------
			for( int i = 0; i < this.SongListFiles.Count; i++ )
			{
				if( !string.IsNullOrEmpty( this.SongListFiles[ i ] ) )
					this.SongListFiles[ i ] = Global.Folder.tファイルパスのマクロを展開する( this.SongListFiles[ i ] );
			}
			//-----------------
			#endregion
			#region [ KitListFiles 要素のシステム/ユーザフォルダパスのマクロを展開する。]
			//-----------------
			for( int i = 0; i < this.KitListFiles.Count; i++ )
			{
				if( !string.IsNullOrEmpty( this.KitListFiles[ i ] ) )
					this.KitListFiles[ i ] = Global.Folder.tファイルパスのマクロを展開する( this.KitListFiles[ i ] );
			}
			//-----------------
			#endregion
		}
		internal void tすべてのAutoPlayを設定する( bool b設定値 )
		{
			Global.Users.Config.AutoPlayLeftCymbal =
				Global.Users.Config.AutoPlayFootPedal =
				Global.Users.Config.AutoPlayHiHat =
				Global.Users.Config.AutoPlaySnare =
				Global.Users.Config.AutoPlayTom1 =
				Global.Users.Config.AutoPlayBass =
				Global.Users.Config.AutoPlayTom2 =
				Global.Users.Config.AutoPlayTom3 =
				Global.Users.Config.AutoPlayRightCymbal = b設定値;
		}
	}
}
